@@ -4,12 +4,10 @@
  *	Declarations of types and functions used to implement button-like
  *	widgets.
  *
- * Copyright (c) 1996-1998 by Sun Microsystems, Inc.
+ * Copyright © 1996-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkButton.h,v 1.16.2.1 2010/03/12 13:02:37 nijtmans Exp $
  */
 
 #ifndef _TKBUTTON
@@ -19,11 +17,6 @@
 #include "tkInt.h"
 #endif
 
-#ifdef BUILD_tk
-# undef TCL_STORAGE_CLASS
-# define TCL_STORAGE_CLASS DLLEXPORT
-#endif
-
 /*
  * Legal values for the "compound" field of TkButton records.
  */
@@ -74,7 +67,7 @@ typedef struct {
     Tcl_Obj *textPtr;		/* Value of -text option: specifies text to
 				 * display in button. */
     int underline;		/* Value of -underline option: specifies index
-				 * of character to underline. < 0 means don't
+				 * of character to underline. INT_MIN means don't
 				 * underline anything. */
     Tcl_Obj *textVarNamePtr;	/* Value of -textvariable option: specifies
 				 * name of variable or NULL. If non-NULL,
@@ -117,10 +110,8 @@ typedef struct {
     Tk_3DBorder activeBorder;	/* Value of -activebackground option: this is
 				 * the color used to draw 3-D border and
 				 * background when widget is active. */
-    Tcl_Obj *borderWidthPtr;	/* Value of -borderWidth option: specifies
+    Tcl_Obj *borderWidthObj;	/* Value of -borderWidth option: specifies
 				 * width of border in pixels. */
-    int borderWidth;		/* Integer value corresponding to
-				 * borderWidthPtr. Always >= 0. */
     int relief;			/* Value of -relief option: specifies 3-d
 				 * effect for border, such as
 				 * TK_RELIEF_RAISED. */
@@ -133,12 +124,10 @@ typedef struct {
 				 * TK_RELIEF_RAISED, to be used when a
 				 * checkbutton or radiobutton without
 				 * indicator is off. */
-    Tcl_Obj *highlightWidthPtr;	/* Value of -highlightthickness option:
+    Tcl_Obj *highlightWidthObj;	/* Value of -highlightthickness option:
 				 * specifies width in pixels of highlight to
 				 * draw around widget when it has the focus.
 				 * <= 0 means don't draw a highlight. */
-    int highlightWidth;		/* Integer value corresponding to
-				 * highlightWidthPtr. Always >= 0. */
     Tk_3DBorder highlightBorder;/* Value of -highlightbackground option:
 				 * specifies background with which to draw 3-D
 				 * default ring and focus highlight area when
@@ -173,26 +162,20 @@ typedef struct {
 				 * disabledFg is NULL. */
     GC copyGC;			/* Used for copying information from an
 				 * off-screen pixmap to the screen. */
-    Tcl_Obj *widthPtr;		/* Value of -width option. */
-    int width;			/* Integer value corresponding to widthPtr. */
-    Tcl_Obj *heightPtr;		/* Value of -height option. */
-    int height;			/* Integer value corresponding to heightPtr. */
-    Tcl_Obj *wrapLengthPtr;	/* Value of -wraplength option: specifies line
+    Tcl_Obj *widthObj;		/* Value of -width option. */
+    Tcl_Obj *heightObj;		/* Value of -height option. */
+    Tcl_Obj *wrapLengthObj;	/* Value of -wraplength option: specifies line
 				 * length (in pixels) at which to wrap onto
 				 * next line. <= 0 means don't wrap except at
 				 * newlines. */
-    int wrapLength;		/* Integer value corresponding to
-				 * wrapLengthPtr. */
-    Tcl_Obj *padXPtr;		/* Value of -padx option: specifies how many
+    Tcl_Obj *padXObj;		/* Value of -padx option: specifies how many
 				 * pixels of extra space to leave on left and
 				 * right of text. Ignored for bitmaps and
 				 * images. */
-    int padX;			/* Integer value corresponding to padXPtr. */
-    Tcl_Obj *padYPtr;		/* Value of -padx option: specifies how many
+    Tcl_Obj *padYObj;		/* Value of -padx option: specifies how many
 				 * pixels of extra space to leave above and
 				 * below text. Ignored for bitmaps and
 				 * images. */
-    int padY;			/* Integer value corresponding to padYPtr. */
     Tk_Anchor anchor;		/* Value of -anchor option: specifies where
 				 * text/bitmap should be displayed inside
 				 * button region. */
@@ -242,7 +225,7 @@ typedef struct {
      * Miscellaneous information:
      */
 
-    Tk_Cursor cursor;		/* Value of -cursor option: if not None,
+    Tk_Cursor cursor;		/* Value of -cursor option: if not NULL,
 				 * specifies current cursor for window. */
     Tcl_Obj *takeFocusPtr;	/* Value of -takefocus option; not used in the
 				 * C code, but used by keyboard traversal
@@ -262,6 +245,12 @@ typedef struct {
 				 * invocataions of the button command. */
     int flags;			/* Various flags; see below for
 				 * definitions. */
+#ifdef BUILD_tk
+    int padX, padY;
+    int borderWidth;
+    int highlightWidth;
+    int width, height, wrapLength;
+#endif
 } TkButton;
 
 /*
@@ -295,31 +284,35 @@ typedef struct {
 #define TRISTATED		(1 << 4)
 
 /*
- * Declaration of variables shared between the files in the button module.
+ * Declaration of button class functions structure
+ * and button/label defaults, for use in optionSpecs.
  */
 
-MODULE_SCOPE Tk_ClassProcs tkpButtonProcs;
+MODULE_SCOPE const Tk_ClassProcs tkpButtonProcs;
+MODULE_SCOPE char tkDefButtonHighlightWidth[TCL_INTEGER_SPACE];
+MODULE_SCOPE char tkDefButtonPadx[TCL_INTEGER_SPACE];
+MODULE_SCOPE char tkDefButtonPady[TCL_INTEGER_SPACE];
 MODULE_SCOPE char tkDefButtonBorderWidth[TCL_INTEGER_SPACE];
+MODULE_SCOPE char tkDefLabelHighlightWidth[TCL_INTEGER_SPACE];
+MODULE_SCOPE char tkDefLabelPadx[TCL_INTEGER_SPACE];
+MODULE_SCOPE char tkDefLabelPady[TCL_INTEGER_SPACE];
 
 /*
  * Declaration of functions used in the implementation of the button widget.
  */
 
 #ifndef TkpButtonSetDefaults
-MODULE_SCOPE void	TkpButtonSetDefaults(Tk_OptionSpec *specPtr);
+MODULE_SCOPE void	TkpButtonSetDefaults(void);
 #endif
-MODULE_SCOPE void	TkButtonWorldChanged(ClientData instanceData);
+MODULE_SCOPE void	TkButtonWorldChanged(void *instanceData);
 MODULE_SCOPE void	TkpComputeButtonGeometry(TkButton *butPtr);
 MODULE_SCOPE TkButton	*TkpCreateButton(Tk_Window tkwin);
 #ifndef TkpDestroyButton
 MODULE_SCOPE void 	TkpDestroyButton(TkButton *butPtr);
 #endif
 #ifndef TkpDisplayButton
-MODULE_SCOPE void	TkpDisplayButton(ClientData clientData);
+MODULE_SCOPE void	TkpDisplayButton(void *clientData);
 #endif
 MODULE_SCOPE int	TkInvokeButton(TkButton *butPtr);
 
-# undef TCL_STORAGE_CLASS
-# define TCL_STORAGE_CLASS DLLIMPORT
-
 #endif /* _TKBUTTON */
