@@ -6,7 +6,7 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * Copyright (c) 2002 Apple Inc.
+ * Copyright © 2002 Apple Inc.
  */
 
 #ifndef _TKENTRY
@@ -45,18 +45,18 @@ typedef struct {
 
     const char *string;		/* Pointer to storage for string;
 				 * NULL-terminated; malloc-ed. */
-    int insertPos;		/* Character index before which next typed
+    Tcl_Size insertPos;		/* Character index before which next typed
 				 * character will be inserted. */
 
     /*
      * Information about what's selected, if any.
      */
 
-    int selectFirst;		/* Character index of first selected character
+    Tcl_Size selectFirst;		/* Character index of first selected character
 				 * (-1 means nothing selected. */
-    int selectLast;		/* Character index just after last selected
+    Tcl_Size selectLast;		/* Character index just after last selected
 				 * character (-1 means nothing selected. */
-    int selectAnchor;		/* Fixed end of selection (i.e. "select to"
+    Tcl_Size selectAnchor;		/* Fixed end of selection (i.e. "select to"
 				 * operation will use this as one end of the
 				 * selection). */
 
@@ -81,7 +81,7 @@ typedef struct {
     Tk_3DBorder readonlyBorder;	/* Used for drawing border around whole window
 				 * in readonly state, plus used for
 				 * background. */
-    int borderWidth;		/* Width of 3-D border around window. */
+    Tcl_Obj *borderWidthObj;	/* Width of 3-D border around window. */
     Tk_Cursor cursor;		/* Current cursor for window, or NULL. */
     int exportSelection;	/* Non-zero means tie internal entry selection
 				 * to X selection. */
@@ -91,23 +91,23 @@ typedef struct {
     XColor *highlightBgColorPtr;/* Color for drawing traversal highlight area
 				 * when highlight is off. */
     XColor *highlightColorPtr;	/* Color for drawing traversal highlight. */
-    int highlightWidth;		/* Width in pixels of highlight to draw around
+    Tcl_Obj *highlightWidthObj;	/* Width in pixels of highlight to draw around
 				 * widget when it has the focus. <= 0 means
 				 * don't draw a highlight. */
     Tk_3DBorder insertBorder;	/* Used to draw vertical bar for insertion
 				 * cursor. */
-    int insertBorderWidth;	/* Width of 3-D border around insert cursor. */
+    Tcl_Obj *insertBorderWidthObj;	/* Width of 3-D border around insert cursor. */
     int insertOffTime;		/* Number of milliseconds cursor should spend
 				 * in "off" state for each blink. */
     int insertOnTime;		/* Number of milliseconds cursor should spend
 				 * in "on" state for each blink. */
-    int insertWidth;		/* Total width of insert cursor. */
+    Tcl_Obj *insertWidthObj;	/* Total width of insert cursor. */
     Tk_Justify justify;		/* Justification to use for text within
 				 * window. */
     int relief;			/* 3-D effect: TK_RELIEF_RAISED, etc. */
     Tk_3DBorder selBorder;	/* Border and background for selected
 				 * characters. */
-    int selBorderWidth;		/* Width of border around selection. */
+    Tcl_Obj *selBorderWidthObj;	/* Width of border around selection. */
     XColor *selFgColorPtr;	/* Foreground color for selected text. */
     int state;			/* Normal or disabled. Entry is read-only when
 				 * disabled. */
@@ -138,13 +138,13 @@ typedef struct {
 				 * malloced memory with the same character
 				 * length as string but whose characters are
 				 * all equal to showChar. */
-    int numBytes;		/* Length of string in bytes. */
-    int numChars;		/* Length of string in characters. Both string
+    Tcl_Size numBytes;		/* Length of string in bytes. */
+    Tcl_Size numChars;		/* Length of string in characters. Both string
 				 * and displayString have the same character
 				 * length, but may have different byte lengths
 				 * due to being made from different UTF-8
 				 * characters. */
-    int numDisplayBytes;	/* Length of displayString in bytes. */
+    Tcl_Size numDisplayBytes;	/* Length of displayString in bytes. */
     int inset;			/* Number of pixels on the left and right
 				 * sides that are taken up by XPAD,
 				 * borderWidth (if any), and highlightWidth
@@ -153,7 +153,7 @@ typedef struct {
     int layoutX, layoutY;	/* Origin for layout. */
     int leftX;			/* X position at which character at leftIndex
 				 * is drawn (varies depending on justify). */
-    int leftIndex;		/* Character index of left-most character
+    Tcl_Size leftIndex;		/* Character index of left-most character
 				 * visible in window. */
     Tcl_TimerToken insertBlinkHandler;
 				/* Timer handler used to blink cursor on and
@@ -173,6 +173,26 @@ typedef struct {
 				 * commands. Malloc'ed. */
     char *invalidCmd;		/* Command called when a validation returns 0
 				 * (successfully fails), defaults to {}. */
+    /*
+     * Fields used in displaying help text if entry value is empty
+     */
+
+    Tk_TextLayout placeholderLayout;/* Cached placeholder text layout information. */
+    char *placeholderString;	/* String value of placeholder. */
+    Tcl_Size placeholderChars;	/* Number of chars in placeholder. */
+    XColor *placeholderColorPtr;/* Color value of placeholder foreground. */
+    GC placeholderGC;		/* For drawing placeholder text. */
+    int placeholderX;		/* Origin for layout. */
+    int placeholderLeftIndex;	/* Character index of left-most character
+				 * visible in window. */
+
+#ifdef BUILD_tk
+    int borderWidth;
+    int highlightWidth;
+    int insertWidth;
+    int insertBorderWidth;
+    int selBorderWidth;
+#endif
 } Entry;
 
 /*
@@ -283,7 +303,7 @@ enum state {
  */
 
 enum selelement {
-    SEL_NONE, SEL_BUTTONDOWN, SEL_BUTTONUP, SEL_NULL, SEL_ENTRY
+    SEL_BUTTONDOWN, SEL_BUTTONUP, SEL_NONE, SEL_NULL, SEL_ENTRY
 };
 
 /*
