@@ -9,17 +9,17 @@
  *	implementation by Paul Mackerras. The -variable option is due to
  *	Henning Schulzrinne. All of these are used with permission.
  *
- * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
- * Copyright (c) 1998-2000 Scriptics Corporation.
+ * Copyright © 1990-1994 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 1998-2000 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
-#include "default.h"
 #include "tkInt.h"
 #include "tkScale.h"
+#include "default.h"
 
 #ifdef _WIN32
 #include "tkWinInt.h"
@@ -36,97 +36,97 @@ static const char *const orientStrings[] = {
 
 static const Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_BORDER, "-activebackground", "activeBackground", "Foreground",
-	DEF_SCALE_ACTIVE_BG_COLOR, -1, Tk_Offset(TkScale, activeBorder),
+	DEF_SCALE_ACTIVE_BG_COLOR, TCL_INDEX_NONE, offsetof(TkScale, activeBorder),
 	0, DEF_SCALE_ACTIVE_BG_MONO, 0},
     {TK_OPTION_BORDER, "-background", "background", "Background",
-	DEF_SCALE_BG_COLOR, -1, Tk_Offset(TkScale, bgBorder),
+	DEF_SCALE_BG_COLOR, TCL_INDEX_NONE, offsetof(TkScale, bgBorder),
 	0, DEF_SCALE_BG_MONO, 0},
     {TK_OPTION_DOUBLE, "-bigincrement", "bigIncrement", "BigIncrement",
-	DEF_SCALE_BIG_INCREMENT, -1, Tk_Offset(TkScale, bigIncrement),
+	DEF_SCALE_BIG_INCREMENT, TCL_INDEX_NONE, offsetof(TkScale, bigIncrement),
 	0, 0, 0},
     {TK_OPTION_SYNONYM, "-bd", NULL, NULL,
-	NULL, 0, -1, 0, "-borderwidth", 0},
+	NULL, 0, TCL_INDEX_NONE, 0, "-borderwidth", 0},
     {TK_OPTION_SYNONYM, "-bg", NULL, NULL,
-	NULL, 0, -1, 0, "-background", 0},
+	NULL, 0, TCL_INDEX_NONE, 0, "-background", 0},
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-	DEF_SCALE_BORDER_WIDTH, -1, Tk_Offset(TkScale, borderWidth),
+	DEF_SCALE_BORDER_WIDTH, offsetof(TkScale, borderWidthObj), TCL_INDEX_NONE,
 	0, 0, 0},
     {TK_OPTION_STRING, "-command", "command", "Command",
-	DEF_SCALE_COMMAND, -1, Tk_Offset(TkScale, command),
+	DEF_SCALE_COMMAND, TCL_INDEX_NONE, offsetof(TkScale, command),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_CURSOR, "-cursor", "cursor", "Cursor",
-	DEF_SCALE_CURSOR, -1, Tk_Offset(TkScale, cursor),
+	DEF_SCALE_CURSOR, TCL_INDEX_NONE, offsetof(TkScale, cursor),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_INT, "-digits", "digits", "Digits",
-	DEF_SCALE_DIGITS, -1, Tk_Offset(TkScale, digits),
+	DEF_SCALE_DIGITS, TCL_INDEX_NONE, offsetof(TkScale, digits),
 	0, 0, 0},
     {TK_OPTION_SYNONYM, "-fg", "foreground", NULL,
-	NULL, 0, -1, 0, "-foreground", 0},
+	NULL, 0, TCL_INDEX_NONE, 0, "-foreground", 0},
     {TK_OPTION_FONT, "-font", "font", "Font",
-	DEF_SCALE_FONT, -1, Tk_Offset(TkScale, tkfont), 0, 0, 0},
+	DEF_SCALE_FONT, TCL_INDEX_NONE, offsetof(TkScale, tkfont), 0, 0, 0},
     {TK_OPTION_COLOR, "-foreground", "foreground", "Foreground",
-	DEF_SCALE_FG_COLOR, -1, Tk_Offset(TkScale, textColorPtr), 0,
-	(ClientData) DEF_SCALE_FG_MONO, 0},
-    {TK_OPTION_DOUBLE, "-from", "from", "From", DEF_SCALE_FROM, -1,
-	Tk_Offset(TkScale, fromValue), 0, 0, 0},
+	DEF_SCALE_FG_COLOR, TCL_INDEX_NONE, offsetof(TkScale, textColorPtr), 0,
+	DEF_SCALE_FG_MONO, 0},
+    {TK_OPTION_DOUBLE, "-from", "from", "From", DEF_SCALE_FROM, TCL_INDEX_NONE,
+	offsetof(TkScale, fromValue), 0, 0, 0},
     {TK_OPTION_BORDER, "-highlightbackground", "highlightBackground",
 	"HighlightBackground", DEF_SCALE_HIGHLIGHT_BG_COLOR,
-	-1, Tk_Offset(TkScale, highlightBorder),
+	TCL_INDEX_NONE, offsetof(TkScale, highlightBorder),
 	0, DEF_SCALE_HIGHLIGHT_BG_MONO, 0},
     {TK_OPTION_COLOR, "-highlightcolor", "highlightColor", "HighlightColor",
-	DEF_SCALE_HIGHLIGHT, -1, Tk_Offset(TkScale, highlightColorPtr),
+	DEF_SCALE_HIGHLIGHT, TCL_INDEX_NONE, offsetof(TkScale, highlightColorPtr),
 	0, 0, 0},
     {TK_OPTION_PIXELS, "-highlightthickness", "highlightThickness",
-	"HighlightThickness", DEF_SCALE_HIGHLIGHT_WIDTH, -1,
-	Tk_Offset(TkScale, highlightWidth), 0, 0, 0},
+	"HighlightThickness", DEF_SCALE_HIGHLIGHT_WIDTH, offsetof(TkScale, highlightWidthObj),
+	TCL_INDEX_NONE, 0, 0, 0},
     {TK_OPTION_STRING, "-label", "label", "Label",
-	DEF_SCALE_LABEL, -1, Tk_Offset(TkScale, label),
+	DEF_SCALE_LABEL, TCL_INDEX_NONE, offsetof(TkScale, label),
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-length", "length", "Length",
-	DEF_SCALE_LENGTH, -1, Tk_Offset(TkScale, length), 0, 0, 0},
+	DEF_SCALE_LENGTH, offsetof(TkScale, lengthObj), TCL_INDEX_NONE, 0, 0, 0},
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient",
-	DEF_SCALE_ORIENT, -1, Tk_Offset(TkScale, orient),
+	DEF_SCALE_ORIENT, TCL_INDEX_NONE, offsetof(TkScale, orient),
 	TK_OPTION_ENUM_VAR, orientStrings, 0},
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
-	DEF_SCALE_RELIEF, -1, Tk_Offset(TkScale, relief), 0, 0, 0},
+	DEF_SCALE_RELIEF, TCL_INDEX_NONE, offsetof(TkScale, relief), 0, 0, 0},
     {TK_OPTION_INT, "-repeatdelay", "repeatDelay", "RepeatDelay",
-	DEF_SCALE_REPEAT_DELAY, -1, Tk_Offset(TkScale, repeatDelay),
+	DEF_SCALE_REPEAT_DELAY, TCL_INDEX_NONE, offsetof(TkScale, repeatDelay),
 	0, 0, 0},
     {TK_OPTION_INT, "-repeatinterval", "repeatInterval", "RepeatInterval",
-	DEF_SCALE_REPEAT_INTERVAL, -1, Tk_Offset(TkScale, repeatInterval),
+	DEF_SCALE_REPEAT_INTERVAL, TCL_INDEX_NONE, offsetof(TkScale, repeatInterval),
 	0, 0, 0},
     {TK_OPTION_DOUBLE, "-resolution", "resolution", "Resolution",
-	DEF_SCALE_RESOLUTION, -1, Tk_Offset(TkScale, resolution),
+	DEF_SCALE_RESOLUTION, TCL_INDEX_NONE, offsetof(TkScale, resolution),
 	0, 0, 0},
     {TK_OPTION_BOOLEAN, "-showvalue", "showValue", "ShowValue",
-	DEF_SCALE_SHOW_VALUE, -1, Tk_Offset(TkScale, showValue),
+	DEF_SCALE_SHOW_VALUE, TCL_INDEX_NONE, offsetof(TkScale, showValue),
 	0, 0, 0},
     {TK_OPTION_PIXELS, "-sliderlength", "sliderLength", "SliderLength",
-	DEF_SCALE_SLIDER_LENGTH, -1, Tk_Offset(TkScale, sliderLength),
+	DEF_SCALE_SLIDER_LENGTH, offsetof(TkScale, sliderLengthObj), TCL_INDEX_NONE,
 	0, 0, 0},
     {TK_OPTION_RELIEF, "-sliderrelief", "sliderRelief", "SliderRelief",
-	DEF_SCALE_SLIDER_RELIEF, -1, Tk_Offset(TkScale, sliderRelief),
+	DEF_SCALE_SLIDER_RELIEF, TCL_INDEX_NONE, offsetof(TkScale, sliderRelief),
 	0, 0, 0},
     {TK_OPTION_STRING_TABLE, "-state", "state", "State",
-	DEF_SCALE_STATE, -1, Tk_Offset(TkScale, state),
+	DEF_SCALE_STATE, TCL_INDEX_NONE, offsetof(TkScale, state),
 	TK_OPTION_ENUM_VAR, tkStateStrings, 0},
     {TK_OPTION_STRING, "-takefocus", "takeFocus", "TakeFocus",
-	DEF_SCALE_TAKE_FOCUS, Tk_Offset(TkScale, takeFocusPtr), -1,
+	DEF_SCALE_TAKE_FOCUS, offsetof(TkScale, takeFocusPtr), TCL_INDEX_NONE,
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_DOUBLE, "-tickinterval", "tickInterval", "TickInterval",
-	DEF_SCALE_TICK_INTERVAL, -1, Tk_Offset(TkScale, tickInterval),
+	DEF_SCALE_TICK_INTERVAL, TCL_INDEX_NONE, offsetof(TkScale, tickInterval),
 	0, 0, 0},
     {TK_OPTION_DOUBLE, "-to", "to", "To",
-	DEF_SCALE_TO, -1, Tk_Offset(TkScale, toValue), 0, 0, 0},
+	DEF_SCALE_TO, TCL_INDEX_NONE, offsetof(TkScale, toValue), 0, 0, 0},
     {TK_OPTION_COLOR, "-troughcolor", "troughColor", "Background",
-	DEF_SCALE_TROUGH_COLOR, -1, Tk_Offset(TkScale, troughColorPtr),
+	DEF_SCALE_TROUGH_COLOR, TCL_INDEX_NONE, offsetof(TkScale, troughColorPtr),
 	0, DEF_SCALE_TROUGH_MONO, 0},
     {TK_OPTION_STRING, "-variable", "variable", "Variable",
-	DEF_SCALE_VARIABLE, Tk_Offset(TkScale, varNamePtr), -1,
+	DEF_SCALE_VARIABLE, offsetof(TkScale, varNamePtr), TCL_INDEX_NONE,
 	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_PIXELS, "-width", "width", "Width",
-	DEF_SCALE_WIDTH, -1, Tk_Offset(TkScale, width), 0, 0, 0},
-    {TK_OPTION_END, NULL, NULL, NULL, NULL, 0, -1, 0, 0, 0}
+	DEF_SCALE_WIDTH, offsetof(TkScale, widthObj), TCL_INDEX_NONE, 0, 0, 0},
+    {TK_OPTION_END, NULL, NULL, NULL, NULL, 0, TCL_INDEX_NONE, 0, 0, 0}
 };
 
 /*
@@ -152,19 +152,17 @@ static void		ComputeFormat(TkScale *scalePtr, int forTicks);
 static void		ComputeScaleGeometry(TkScale *scalePtr);
 static int		ConfigureScale(Tcl_Interp *interp, TkScale *scalePtr,
 			    int objc, Tcl_Obj *const objv[]);
-static void		DestroyScale(char *memPtr);
+static void		DestroyScale(void *memPtr);
 static double		MaxTickRoundingError(TkScale *scalePtr,
 			    double tickResolution);
-static void		ScaleCmdDeletedProc(ClientData clientData);
-static void		ScaleEventProc(ClientData clientData,
+static void		ScaleCmdDeletedProc(void *clientData);
+static void		ScaleEventProc(void *clientData,
 			    XEvent *eventPtr);
-static char *		ScaleVarProc(ClientData clientData,
+static char *		ScaleVarProc(void *clientData,
 			    Tcl_Interp *interp, const char *name1,
 			    const char *name2, int flags);
-static int		ScaleWidgetObjCmd(ClientData clientData,
-			    Tcl_Interp *interp, int objc,
-			    Tcl_Obj *const objv[]);
-static void		ScaleWorldChanged(ClientData instanceData);
+static Tcl_ObjCmdProc ScaleWidgetObjCmd;
+static void		ScaleWorldChanged(void *instanceData);
 static void		ScaleSetVariable(TkScale *scalePtr);
 
 /*
@@ -239,7 +237,7 @@ ScaleRound(
 
 int
 Tk_ScaleObjCmd(
-    ClientData clientData,	/* NULL. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument values. */
@@ -283,8 +281,8 @@ Tk_ScaleObjCmd(
 	    scalePtr, ScaleCmdDeletedProc);
     scalePtr->optionTable	= optionTable;
     scalePtr->orient		= ORIENT_VERTICAL;
-    scalePtr->width		= 0;
-    scalePtr->length		= 0;
+    scalePtr->widthObj		= NULL;
+    scalePtr->lengthObj		= NULL;
     scalePtr->value		= 0.0;
     scalePtr->varNamePtr	= NULL;
     scalePtr->fromValue		= 0.0;
@@ -299,7 +297,7 @@ Tk_ScaleObjCmd(
     scalePtr->label		= NULL;
     scalePtr->labelLength	= 0;
     scalePtr->state		= STATE_NORMAL;
-    scalePtr->borderWidth	= 0;
+    scalePtr->borderWidthObj	= NULL;
     scalePtr->bgBorder		= NULL;
     scalePtr->activeBorder	= NULL;
     scalePtr->sliderRelief	= TK_RELIEF_RAISED;
@@ -310,11 +308,11 @@ Tk_ScaleObjCmd(
     scalePtr->textColorPtr	= NULL;
     scalePtr->textGC		= NULL;
     scalePtr->relief		= TK_RELIEF_FLAT;
-    scalePtr->highlightWidth	= 0;
+    scalePtr->highlightWidthObj	= NULL;
     scalePtr->highlightBorder	= NULL;
     scalePtr->highlightColorPtr	= NULL;
     scalePtr->inset		= 0;
-    scalePtr->sliderLength	= 0;
+    scalePtr->sliderLengthObj	= NULL;
     scalePtr->showValue		= 0;
     scalePtr->horizLabelY	= 0;
     scalePtr->horizValueY	= 0;
@@ -329,12 +327,14 @@ Tk_ScaleObjCmd(
     scalePtr->takeFocusPtr	= NULL;
     scalePtr->flags		= NEVER_SET;
 
+
+
     Tk_SetClassProcs(scalePtr->tkwin, &scaleClass, scalePtr);
     Tk_CreateEventHandler(scalePtr->tkwin,
 	    ExposureMask|StructureNotifyMask|FocusChangeMask,
 	    ScaleEventProc, scalePtr);
 
-    if ((Tk_InitOptions(interp, (char *) scalePtr, optionTable, tkwin)
+    if ((Tk_InitOptions(interp, scalePtr, optionTable, tkwin)
 	    != TCL_OK) ||
 	    (ConfigureScale(interp, scalePtr, objc - 2, objv + 2) != TCL_OK)) {
 	Tk_DestroyWindow(scalePtr->tkwin);
@@ -347,7 +347,7 @@ Tk_ScaleObjCmd(
 
     scalePtr->flags &= ~INVOKE_COMMAND;
 
-    Tcl_SetObjResult(interp, TkNewWindowObj(scalePtr->tkwin));
+    Tcl_SetObjResult(interp, Tk_NewWindowObj(scalePtr->tkwin));
     return TCL_OK;
 }
 
@@ -371,12 +371,12 @@ Tk_ScaleObjCmd(
 
 static int
 ScaleWidgetObjCmd(
-    ClientData clientData,	/* Information about scale widget. */
+    void *clientData,	/* Information about scale widget. */
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const objv[])	/* Argument strings. */
 {
-    TkScale *scalePtr = clientData;
+    TkScale *scalePtr = (TkScale *)clientData;
     Tcl_Obj *objPtr;
     int index, result;
 
@@ -397,7 +397,7 @@ ScaleWidgetObjCmd(
 	    Tcl_WrongNumArgs(interp, 1, objv, "cget option");
 	    goto error;
 	}
-	objPtr = Tk_GetOptionValue(interp, (char *) scalePtr,
+	objPtr = Tk_GetOptionValue(interp, scalePtr,
 		scalePtr->optionTable, objv[2], scalePtr->tkwin);
 	if (objPtr == NULL) {
 	    goto error;
@@ -406,7 +406,7 @@ ScaleWidgetObjCmd(
 	break;
     case COMMAND_CONFIGURE:
 	if (objc <= 3) {
-	    objPtr = Tk_GetOptionInfo(interp, (char *) scalePtr,
+	    objPtr = Tk_GetOptionInfo(interp, scalePtr,
 		    scalePtr->optionTable,
 		    (objc == 3) ? objv[2] : NULL, scalePtr->tkwin);
 	    if (objPtr == NULL) {
@@ -418,7 +418,7 @@ ScaleWidgetObjCmd(
 	}
 	break;
     case COMMAND_COORDS: {
-	int x, y;
+	int x, y, width, borderWidth;
 	double value;
 	Tcl_Obj *coords[2];
 
@@ -433,17 +433,19 @@ ScaleWidgetObjCmd(
 	} else {
 	    value = scalePtr->value;
 	}
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->widthObj, &width);
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
 	if (scalePtr->orient == ORIENT_VERTICAL) {
-	    x = scalePtr->vertTroughX + scalePtr->width/2
-		    + scalePtr->borderWidth;
+	    x = scalePtr->vertTroughX + width/2
+		    + borderWidth;
 	    y = TkScaleValueToPixel(scalePtr, value);
 	} else {
 	    x = TkScaleValueToPixel(scalePtr, value);
-	    y = scalePtr->horizTroughY + scalePtr->width/2
-		    + scalePtr->borderWidth;
+	    y = scalePtr->horizTroughY + width/2
+		    + borderWidth;
 	}
-	coords[0] = Tcl_NewIntObj(x);
-	coords[1] = Tcl_NewIntObj(y);
+	coords[0] = Tcl_NewWideIntObj(x);
+	coords[1] = Tcl_NewWideIntObj(y);
 	Tcl_SetObjResult(interp, Tcl_NewListObj(2, coords));
 	break;
     }
@@ -484,7 +486,7 @@ ScaleWidgetObjCmd(
 	case SLIDER:	zone = "slider";  break;
 	case TROUGH2:	zone = "trough2"; break;
 	}
-	Tcl_SetObjResult(interp, Tcl_NewStringObj(zone, -1));
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(zone, TCL_INDEX_NONE));
 	break;
     }
     case COMMAND_SET: {
@@ -531,7 +533,7 @@ ScaleWidgetObjCmd(
 
 static void
 DestroyScale(
-    char *memPtr)	/* Info about scale widget. */
+    void *memPtr)	/* Info about scale widget. */
 {
     TkScale *scalePtr = (TkScale *) memPtr;
 
@@ -561,7 +563,7 @@ DestroyScale(
     if (scalePtr->textGC != NULL) {
 	Tk_FreeGC(scalePtr->display, scalePtr->textGC);
     }
-    Tk_FreeConfigOptions((char *) scalePtr, scalePtr->optionTable,
+    Tk_FreeConfigOptions(scalePtr, scalePtr->optionTable,
 	    scalePtr->tkwin);
     scalePtr->tkwin = NULL;
     TkpDestroyScale(scalePtr);
@@ -597,7 +599,7 @@ ConfigureScale(
 {
     Tk_SavedOptions savedOptions;
     Tcl_Obj *errorResult = NULL;
-    int error;
+    int error, highlightWidth, borderWidth;
     double varValue;
 
     /*
@@ -616,7 +618,7 @@ ConfigureScale(
 	     * First pass: set options to new values.
 	     */
 
-	    if (Tk_SetOptions(interp, (char *) scalePtr,
+	    if (Tk_SetOptions(interp, scalePtr,
 		    scalePtr->optionTable, objc, objv, scalePtr->tkwin,
 		    &savedOptions, NULL) != TCL_OK) {
 		continue;
@@ -649,10 +651,10 @@ ConfigureScale(
 	    }
 	}
 
-        /*
-         * The fromValue shall not be rounded to the resolution, but the
-         * toValue and tickInterval do.
-         */
+	/*
+	 * The fromValue shall not be rounded to the resolution, but the
+	 * toValue and tickInterval do.
+	 */
 
 	scalePtr->toValue = TkRoundValueToResolution(scalePtr, scalePtr->toValue);
 	scalePtr->tickInterval = TkRoundIntervalToResolution(scalePtr,
@@ -671,14 +673,21 @@ ConfigureScale(
 	ComputeFormat(scalePtr, 0);
 	ComputeFormat(scalePtr, 1);
 
-	scalePtr->labelLength = scalePtr->label ? (int)strlen(scalePtr->label) : 0;
+	scalePtr->labelLength = scalePtr->label ? strlen(scalePtr->label) : 0;
 
 	Tk_SetBackgroundFromBorder(scalePtr->tkwin, scalePtr->bgBorder);
 
-	if (scalePtr->highlightWidth < 0) {
-	    scalePtr->highlightWidth = 0;
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->highlightWidthObj, &highlightWidth);
+	if (highlightWidth < 0) {
+	    highlightWidth = 0;
+		if (scalePtr->highlightWidthObj) {
+		    Tcl_DecrRefCount(scalePtr->highlightWidthObj);
+		}
+		scalePtr->highlightWidthObj = Tcl_NewIntObj(0);
+		Tcl_IncrRefCount(scalePtr->highlightWidthObj);
 	}
-	scalePtr->inset = scalePtr->highlightWidth + scalePtr->borderWidth;
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+	scalePtr->inset = highlightWidth + borderWidth;
 	break;
     }
     if (!error) {
@@ -714,9 +723,9 @@ ConfigureScale(
 	} else {
 	    char varString[TCL_DOUBLE_SPACE], scaleString[TCL_DOUBLE_SPACE];
 
-            Tcl_PrintDouble(NULL, varValue, varString);
-            Tcl_PrintDouble(NULL, scalePtr->value, scaleString);
-            if (strcmp(varString, scaleString)) {
+	    Tcl_PrintDouble(NULL, varValue, varString);
+	    Tcl_PrintDouble(NULL, scalePtr->value, scaleString);
+	    if (strcmp(varString, scaleString)) {
 		ScaleSetVariable(scalePtr);
 	    }
 	}
@@ -754,11 +763,12 @@ ConfigureScale(
 
 static void
 ScaleWorldChanged(
-    ClientData instanceData)	/* Information about widget. */
+    void *instanceData)	/* Information about widget. */
 {
     XGCValues gcValues;
     GC gc;
-    TkScale *scalePtr = instanceData;
+    TkScale *scalePtr = (TkScale *)instanceData;
+    int highlightWidth, borderWidth;
 
     gcValues.foreground = scalePtr->troughColorPtr->pixel;
     gc = Tk_GetGC(scalePtr->tkwin, GCForeground, &gcValues);
@@ -780,7 +790,9 @@ ScaleWorldChanged(
 	scalePtr->copyGC = Tk_GetGC(scalePtr->tkwin, GCGraphicsExposures,
 		&gcValues);
     }
-    scalePtr->inset = scalePtr->highlightWidth + scalePtr->borderWidth;
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->highlightWidthObj, &highlightWidth);
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+    scalePtr->inset = highlightWidth + borderWidth;
 
     /*
      * Recompute display-related information, and let the geometry manager
@@ -933,10 +945,12 @@ ComputeFormat(
 		 * value between adjacent pixels and use it for the least
 		 * significant digit.
 		 */
+		int length;
 
 		x = fabs(scalePtr->fromValue - scalePtr->toValue);
-		if (scalePtr->length > 0) {
-		    x /= scalePtr->length;
+	    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->lengthObj, &length);
+		if (length > 0) {
+		    x /= length;
 		}
 		if (x > 0) {
 		    leastSigDigit = ScaleDigit(x);
@@ -1013,6 +1027,7 @@ ComputeScaleGeometry(
     char valueString[TCL_DOUBLE_SPACE];
     int tmp, valuePixels, tickPixels, x, y, extraSpace;
     Tk_FontMetrics fm;
+    int length, width, borderWidth;
 
     Tk_GetFontMetrics(scalePtr->tkfont, &fm);
     scalePtr->fontHeight = fm.linespace + SPACING;
@@ -1040,13 +1055,16 @@ ComputeScaleGeometry(
 	}
 	y += extraSpace;
 	scalePtr->horizTroughY = y;
-	y += scalePtr->width + 2*scalePtr->borderWidth;
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->lengthObj, &length);
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->widthObj, &width);
+	Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+	y += width + 2 * borderWidth;
 	if (scalePtr->tickInterval != 0) {
 	    scalePtr->horizTickY = y + SPACING;
 	    y += scalePtr->fontHeight + SPACING;
 	}
 	Tk_GeometryRequest(scalePtr->tkwin,
-		scalePtr->length + 2*scalePtr->inset, y + scalePtr->inset);
+		length + 2*scalePtr->inset, y + scalePtr->inset);
 	Tk_SetInternalBorder(scalePtr->tkwin, scalePtr->inset);
 	return;
     }
@@ -1058,14 +1076,14 @@ ComputeScaleGeometry(
      */
 
     if (snprintf(valueString, TCL_DOUBLE_SPACE, scalePtr->valueFormat,
-            scalePtr->fromValue) < 0) {
-        valueString[TCL_DOUBLE_SPACE - 1] = '\0';
+	    scalePtr->fromValue) < 0) {
+	valueString[TCL_DOUBLE_SPACE - 1] = '\0';
     }
     valuePixels = Tk_TextWidth(scalePtr->tkfont, valueString, -1);
 
     if (snprintf(valueString, TCL_DOUBLE_SPACE, scalePtr->valueFormat,
-            scalePtr->toValue) < 0) {
-        valueString[TCL_DOUBLE_SPACE - 1] = '\0';
+	    scalePtr->toValue) < 0) {
+	valueString[TCL_DOUBLE_SPACE - 1] = '\0';
     }
     tmp = Tk_TextWidth(scalePtr->tkfont, valueString, -1);
     if (valuePixels < tmp) {
@@ -1077,14 +1095,14 @@ ComputeScaleGeometry(
      */
 
     if (snprintf(valueString, TCL_DOUBLE_SPACE, scalePtr->tickFormat,
-            scalePtr->fromValue) < 0) {
-        valueString[TCL_DOUBLE_SPACE - 1] = '\0';
+	    scalePtr->fromValue) < 0) {
+	valueString[TCL_DOUBLE_SPACE - 1] = '\0';
     }
     tickPixels = Tk_TextWidth(scalePtr->tkfont, valueString, -1);
 
     if (snprintf(valueString, TCL_DOUBLE_SPACE, scalePtr->tickFormat,
-            scalePtr->toValue) < 0) {
-        valueString[TCL_DOUBLE_SPACE - 1] = '\0';
+	    scalePtr->toValue) < 0) {
+	valueString[TCL_DOUBLE_SPACE - 1] = '\0';
     }
     tmp = Tk_TextWidth(scalePtr->tkfont, valueString, -1);
     if (tickPixels < tmp) {
@@ -1115,7 +1133,9 @@ ComputeScaleGeometry(
 	scalePtr->vertValueRightX = x;
     }
     scalePtr->vertTroughX = x;
-    x += 2*scalePtr->borderWidth + scalePtr->width;
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->widthObj, &width);
+    x += 2 * borderWidth + width;
     if (scalePtr->labelLength == 0) {
 	scalePtr->vertLabelX = 0;
     } else {
@@ -1124,8 +1144,9 @@ ComputeScaleGeometry(
 	    + Tk_TextWidth(scalePtr->tkfont, scalePtr->label,
 		    scalePtr->labelLength);
     }
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->lengthObj, &length);
     Tk_GeometryRequest(scalePtr->tkwin, x + scalePtr->inset,
-	    scalePtr->length + 2*scalePtr->inset);
+	    length + 2*scalePtr->inset);
     Tk_SetInternalBorder(scalePtr->tkwin, scalePtr->inset);
 }
 
@@ -1149,10 +1170,11 @@ ComputeScaleGeometry(
 
 static void
 ScaleEventProc(
-    ClientData clientData,	/* Information about window. */
+    void *clientData,	/* Information about window. */
     XEvent *eventPtr)		/* Information about event. */
 {
-    TkScale *scalePtr = clientData;
+    TkScale *scalePtr = (TkScale *)clientData;
+    int highlightWidth;
 
     if ((eventPtr->type == Expose) && (eventPtr->xexpose.count == 0)) {
 	TkEventuallyRedrawScale(scalePtr, REDRAW_ALL);
@@ -1164,14 +1186,16 @@ ScaleEventProc(
     } else if (eventPtr->type == FocusIn) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    scalePtr->flags |= GOT_FOCUS;
-	    if (scalePtr->highlightWidth > 0) {
+	    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->highlightWidthObj, &highlightWidth);
+	    if (highlightWidth > 0) {
 		TkEventuallyRedrawScale(scalePtr, REDRAW_ALL);
 	    }
 	}
     } else if (eventPtr->type == FocusOut) {
 	if (eventPtr->xfocus.detail != NotifyInferior) {
 	    scalePtr->flags &= ~GOT_FOCUS;
-	    if (scalePtr->highlightWidth > 0) {
+	    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->highlightWidthObj, &highlightWidth);
+	    if (highlightWidth > 0) {
 		TkEventuallyRedrawScale(scalePtr, REDRAW_ALL);
 	    }
 	}
@@ -1198,9 +1222,9 @@ ScaleEventProc(
 
 static void
 ScaleCmdDeletedProc(
-    ClientData clientData)	/* Pointer to widget record for widget. */
+    void *clientData)	/* Pointer to widget record for widget. */
 {
-    TkScale *scalePtr = clientData;
+    TkScale *scalePtr = (TkScale *)clientData;
     Tk_Window tkwin = scalePtr->tkwin;
 
     /*
@@ -1279,7 +1303,7 @@ TkRoundValueToResolution(
     double value)		/* Value to round. */
 {
     return TkRoundIntervalToResolution(scalePtr, value - scalePtr->fromValue)
-            + scalePtr->fromValue;
+	    + scalePtr->fromValue;
 }
 
 double
@@ -1296,13 +1320,13 @@ TkRoundIntervalToResolution(
     rounded = scalePtr->resolution * tick;
     rem = value - rounded;
     if (rem < 0) {
-        if (rem <= -scalePtr->resolution/2) {
-            rounded = (tick - 1.0) * scalePtr->resolution;
-        }
+	if (rem <= -scalePtr->resolution/2) {
+	    rounded = (tick - 1.0) * scalePtr->resolution;
+	}
     } else {
-        if (rem >= scalePtr->resolution/2) {
-            rounded = (tick + 1.0) * scalePtr->resolution;
-        }
+	if (rem >= scalePtr->resolution/2) {
+	    rounded = (tick + 1.0) * scalePtr->resolution;
+	}
     }
     return rounded;
 }
@@ -1326,16 +1350,15 @@ TkRoundIntervalToResolution(
  *----------------------------------------------------------------------
  */
 
-    /* ARGSUSED */
 static char *
 ScaleVarProc(
-    ClientData clientData,	/* Information about button. */
+    void *clientData,	/* Information about button. */
     Tcl_Interp *interp,		/* Interpreter containing variable. */
-    const char *name1,		/* Name of variable. */
-    const char *name2,		/* Second part of variable name. */
+    TCL_UNUSED(const char *),	/* Name of variable. */
+    TCL_UNUSED(const char *),	/* Second part of variable name. */
     int flags)			/* Information about what happened. */
 {
-    TkScale *scalePtr = clientData;
+    TkScale *scalePtr = (TkScale *)clientData;
     const char *resultStr;
     double value;
     Tcl_Obj *valuePtr;
@@ -1347,27 +1370,27 @@ ScaleVarProc(
      */
 
     if (flags & TCL_TRACE_UNSETS) {
-        if (!Tcl_InterpDeleted(interp) && scalePtr->varNamePtr) {
-            ClientData probe = NULL;
+	if (!Tcl_InterpDeleted(interp) && scalePtr->varNamePtr) {
+	    void *probe = NULL;
 
-            do {
-                probe = Tcl_VarTraceInfo(interp,
-                        Tcl_GetString(scalePtr->varNamePtr),
-                        TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
-                        ScaleVarProc, probe);
-                if (probe == (ClientData)scalePtr) {
-                    break;
-                }
-            } while (probe);
-            if (probe) {
-                /*
-                 * We were able to fetch the unset trace for our
-                 * varNamePtr, which means it is not unset and not
-                 * the cause of this unset trace. Instead some outdated
-                 * former variable must be, and we should ignore it.
-                 */
-                return NULL;
-            }
+	    do {
+		probe = Tcl_VarTraceInfo(interp,
+			Tcl_GetString(scalePtr->varNamePtr),
+			TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
+			ScaleVarProc, probe);
+		if (probe == (void *)scalePtr) {
+		    break;
+		}
+	    } while (probe);
+	    if (probe) {
+		/*
+		 * We were able to fetch the unset trace for our
+		 * varNamePtr, which means it is not unset and not
+		 * the cause of this unset trace. Instead some outdated
+		 * former variable must be, and we should ignore it.
+		 */
+		return NULL;
+	    }
 	    Tcl_TraceVar2(interp, Tcl_GetString(scalePtr->varNamePtr),
 		    NULL, TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
 		    ScaleVarProc, clientData);
@@ -1391,7 +1414,7 @@ ScaleVarProc(
 	    TCL_GLOBAL_ONLY);
     result = Tcl_GetDoubleFromObj(interp, valuePtr, &value);
     if (result != TCL_OK) {
-	resultStr = "can't assign non-numeric value to scale variable";
+	resultStr = "cannot assign a non-numeric value to a scale variable";
 	ScaleSetVariable(scalePtr);
     } else {
 	scalePtr->value = TkRoundValueToResolution(scalePtr, value);
@@ -1493,13 +1516,13 @@ ScaleSetVariable(
     if (scalePtr->varNamePtr != NULL) {
 	char string[TCL_DOUBLE_SPACE];
 
-        if (snprintf(string, TCL_DOUBLE_SPACE, scalePtr->valueFormat,
-                scalePtr->value) < 0) {
-            string[TCL_DOUBLE_SPACE - 1] = '\0';
-        }
+	if (snprintf(string, TCL_DOUBLE_SPACE, scalePtr->valueFormat,
+		scalePtr->value) < 0) {
+	    string[TCL_DOUBLE_SPACE - 1] = '\0';
+	}
 	scalePtr->flags |= SETTING_VAR;
 	Tcl_ObjSetVar2(scalePtr->interp, scalePtr->varNamePtr, NULL,
-		Tcl_NewStringObj(string, -1), TCL_GLOBAL_ONLY);
+		Tcl_NewStringObj(string, TCL_INDEX_NONE), TCL_GLOBAL_ONLY);
 	scalePtr->flags &= ~SETTING_VAR;
     }
 }
@@ -1528,14 +1551,17 @@ TkScalePixelToValue(
     int x, int y)		/* Coordinates of point within window. */
 {
     double value, pixelRange;
+    int borderWidth, sliderLength;
 
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->sliderLengthObj, &sliderLength);
     if (scalePtr->orient == ORIENT_VERTICAL) {
-	pixelRange = Tk_Height(scalePtr->tkwin) - scalePtr->sliderLength
-		- 2*scalePtr->inset - 2*scalePtr->borderWidth;
+	pixelRange = Tk_Height(scalePtr->tkwin) - sliderLength
+		- 2 * scalePtr->inset - 2 * borderWidth;
 	value = y;
     } else {
-	pixelRange = Tk_Width(scalePtr->tkwin) - scalePtr->sliderLength
-		- 2*scalePtr->inset - 2*scalePtr->borderWidth;
+	pixelRange = Tk_Width(scalePtr->tkwin) - sliderLength
+		- 2 * scalePtr->inset - 2 * borderWidth;
 	value = x;
     }
 
@@ -1547,8 +1573,8 @@ TkScalePixelToValue(
 
 	return scalePtr->value;
     }
-    value -= scalePtr->sliderLength/2 + scalePtr->inset
-	    + scalePtr->borderWidth;
+    value -= sliderLength/2 + scalePtr->inset
+	    + borderWidth;
     value /= pixelRange;
     if (value < 0) {
 	value = 0;
@@ -1587,11 +1613,14 @@ TkScaleValueToPixel(
 {
     int y, pixelRange;
     double valueRange;
+    int borderWidth, sliderLength;
 
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->borderWidthObj, &borderWidth);
+    Tk_GetPixelsFromObj(NULL, scalePtr->tkwin, scalePtr->sliderLengthObj, &sliderLength);
     valueRange = scalePtr->toValue - scalePtr->fromValue;
     pixelRange = ((scalePtr->orient == ORIENT_VERTICAL)
 	    ? Tk_Height(scalePtr->tkwin) : Tk_Width(scalePtr->tkwin))
-	- scalePtr->sliderLength - 2*scalePtr->inset - 2*scalePtr->borderWidth;
+	- sliderLength - 2 * scalePtr->inset - 2 * borderWidth;
     if (valueRange == 0) {
 	y = 0;
     } else {
@@ -1603,7 +1632,7 @@ TkScaleValueToPixel(
 	    y = pixelRange;
 	}
     }
-    y += scalePtr->sliderLength/2 + scalePtr->inset + scalePtr->borderWidth;
+    y += sliderLength / 2 + scalePtr->inset + borderWidth;
     return y;
 }
 
