@@ -4,8 +4,8 @@
  *	Declarations of types and functions used to implement the scale
  *	widget.
  *
- * Copyright (c) 1996 by Sun Microsystems, Inc.
- * Copyright (c) 1999-2000 by Scriptics Corporation.
+ * Copyright © 1996 Sun Microsystems, Inc.
+ * Copyright © 1999-2000 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -53,9 +53,9 @@ typedef struct TkScale {
 				 * available for this widget. */
     enum orient orient;		/* Orientation for window (vertical or
 				 * horizontal). */
-    int width;			/* Desired narrow dimension of scale, in
+    Tcl_Obj *widthObj;		/* Desired narrow dimension of scale, in
 				 * pixels. */
-    int length;			/* Desired long dimension of scale, in
+    Tcl_Obj *lengthObj;	/* Desired long dimension of scale, in
 				 * pixels. */
     double value;		/* Current value of scale. */
     Tcl_Obj *varNamePtr;	/* Name of variable or NULL. If non-NULL,
@@ -73,8 +73,10 @@ typedef struct TkScale {
 				 * values. 0 means we get to choose the number
 				 * based on resolution and/or the range of the
 				 * scale. */
-    char format[10];		/* Sprintf conversion specifier computed from
+    char valueFormat[16];	/* Snprintf conversion specifier computed from
 				 * digits and other information. */
+    char tickFormat[16];	/* Snprintf conversion specifier computed from
+				 * tick interval. */
     double bigIncrement;	/* Amount to use for large increments to scale
 				 * value. (0 means we pick a value). */
     char *command;		/* Command prefix to use when invoking Tcl
@@ -85,7 +87,7 @@ typedef struct TkScale {
     int repeatInterval;		/* Interval between autorepeats (in ms). */
     char *label;		/* Label to display above or to right of
 				 * scale; NULL means don't display a label. */
-    int labelLength;		/* Number of non-NULL chars. in label. */
+    Tcl_Size labelLength;	/* Number of non-NULL chars. in label. */
     enum state state;		/* Values are active, normal, or disabled.
 				 * Value of scale cannot be changed when
 				 * disabled. */
@@ -94,7 +96,7 @@ typedef struct TkScale {
      * Information used when displaying widget:
      */
 
-    int borderWidth;		/* Width of 3-D border around window. */
+    Tcl_Obj *borderWidthObj;	/* Width of 3-D border around window. */
     Tk_3DBorder bgBorder;	/* Used for drawing slider and other
 				 * background areas. */
     Tk_3DBorder activeBorder;	/* For drawing the slider when active. */
@@ -108,7 +110,7 @@ typedef struct TkScale {
     GC textGC;			/* GC for drawing text in normal mode. */
     int relief;			/* Indicates whether window as a whole is
 				 * raised, sunken, or flat. */
-    int highlightWidth;		/* Width in pixels of highlight to draw around
+    Tcl_Obj *highlightWidthObj;	/* Width in pixels of highlight to draw around
 				 * widget when it has the focus. <= 0 means
 				 * don't draw a highlight. */
     Tk_3DBorder highlightBorder;/* Value of -highlightbackground option:
@@ -121,7 +123,7 @@ typedef struct TkScale {
 				 * Indicates how much interior stuff must be
 				 * offset from outside edges to leave room for
 				 * borders. */
-    int sliderLength;		/* Length of slider, measured in pixels along
+    Tcl_Obj *sliderLengthObj;	/* Length of slider, measured in pixels along
 				 * long dimension of scale. */
     int showValue;		/* Non-zero means to display the scale value
 				 * below or to the left of the slider; zero
@@ -151,7 +153,7 @@ typedef struct TkScale {
      */
 
     int fontHeight;		/* Height of scale font. */
-    Tk_Cursor cursor;		/* Current cursor for window, or None. */
+    Tk_Cursor cursor;		/* Current cursor for window, or NULL. */
     Tcl_Obj *takeFocusPtr;	/* Value of -takefocus option; not used in the
 				 * C code, but used by keyboard traversal
 				 * scripts. May be NULL. */
@@ -215,20 +217,23 @@ typedef struct TkScale {
 #define SPACING 2
 
 /*
- * How many characters of space to provide when formatting the scale's value:
+ * The tick values are all displayed with the same number of decimal places.
+ * This number of decimal places is such that the displayed values are all
+ * accurate to within the following proportion of a tick interval.
  */
 
-#define PRINT_CHARS 150
+#define TICK_VALUES_DISPLAY_ACCURACY 0.2
 
 /*
  * Declaration of procedures used in the implementation of the scale widget.
  */
 
 MODULE_SCOPE void	TkEventuallyRedrawScale(TkScale *scalePtr, int what);
-MODULE_SCOPE double	TkRoundToResolution(TkScale *scalePtr, double value);
+MODULE_SCOPE double	TkRoundValueToResolution(TkScale *scalePtr, double value);
+MODULE_SCOPE double	TkRoundIntervalToResolution(TkScale *scalePtr, double value);
 MODULE_SCOPE TkScale *	TkpCreateScale(Tk_Window tkwin);
 MODULE_SCOPE void	TkpDestroyScale(TkScale *scalePtr);
-MODULE_SCOPE void	TkpDisplayScale(ClientData clientData);
+MODULE_SCOPE void	TkpDisplayScale(void *clientData);
 MODULE_SCOPE int	TkpScaleElement(TkScale *scalePtr, int x, int y);
 MODULE_SCOPE void	TkScaleSetValue(TkScale *scalePtr, double value,
 			    int setVar, int invokeCommand);
