@@ -3,7 +3,7 @@
  *
  *	This file contains the initializers for the Tk stub vectors.
  *
- * Copyright (c) 1998-1999 Scriptics Corporation.
+ * Copyright © 1998-1999 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -25,60 +25,40 @@
 /* we could have used _TKMACINT */
 #include "tkMacOSXInt.h"
 #include "tkMacOSXPrivate.h"
+#else
+#   define Tk_ClipDrawableToRect 0
 #endif
 
 /* TODO: These ought to come in some other way */
 #include "tkPlatDecls.h"
 #include "tkIntXlibDecls.h"
 
-static const TkIntStubs tkIntStubs;
 MODULE_SCOPE const TkStubs tkStubs;
 
 /*
  * Remove macro that might interfere with the definition below.
  */
 
-#undef Tk_MainEx
-#undef XVisualIDFromVisual
-#undef XSynchronize
-#undef XUngrabServer
-#undef XNoOp
-#undef XGrabServer
-#undef XFree
-#undef XFlush
-#undef Tk_FreeStyleFromObj
-#define TkUnusedStubEntry 0
+#undef TkPutImage
+#undef XPutImage
 
-#define TkpCmapStressed_ TkpCmapStressed
-#define TkpSync_ TkpSync
-#define TkUnixContainerId_ TkUnixContainerId
-#define TkUnixDoOneXEvent_ TkUnixDoOneXEvent
-#define TkUnixSetMenubar_ TkUnixSetMenubar
-#define TkWmCleanup_ TkWmCleanup
-#define TkSendCleanup_ TkSendCleanup
-#define TkpTestsendCmd_ TkpTestsendCmd
-#define TkGenWMConfigureEvent_ TkGenWMConfigureEvent
-#define TkGenerateActivateEvents_ TkGenerateActivateEvents
-
-#if !defined(MAC_OSX_TK) && defined(MAC_OSX_TCL)
-#   undef TkpWillDrawWidget
-#   undef TkpRedrawWidget
+#if !defined(MAC_OSX_TK)
 static int
 doNothing(void)
 {
     /* dummy implementation, no need to do anything */
     return 0;
 }
+#   undef TkpWillDrawWidget
+#   undef TkpRedrawWidget
+#   undef TkpDefineNativeBitmaps
+#   undef TkpCreateNativeBitmap
+#   undef TkpGetNativeAppBitmap
 #   define TkpWillDrawWidget ((int (*)(Tk_Window))(void *)doNothing)
 #   define TkpRedrawWidget ((void (*)(Tk_Window))(void *)doNothing)
-#endif
-
-#if defined(MAC_OSX_TK)
-#   define Tk_MacOSXGetNSWindowForDrawable TkMacOSXDrawable
-#   define Tk_MacOSXGetCGContextForDrawable GetCGContextForDrawable
-static void *GetCGContextForDrawable(Drawable d) {
-	return TkMacOSXGetCGContextForDrawable(d);
-}
+#   define TkpDefineNativeBitmaps ((void (*)(void))(void *)doNothing)
+#   define TkpCreateNativeBitmap ((Pixmap (*)(Display *, const void *))(void *)doNothing)
+#   define TkpGetNativeAppBitmap ((Pixmap (*)(Display *, const char*, int *, int *))(void *)doNothing)
 #endif
 
 #ifdef _WIN32
@@ -86,12 +66,16 @@ static void *GetCGContextForDrawable(Drawable d) {
 int
 TkpCmapStressed(Tk_Window tkwin, Colormap colormap)
 {
+    (void)tkwin;
+    (void)colormap;
+
     /* dummy implementation, no need to do anything */
     return 0;
 }
 void
 TkpSync(Display *display)
 {
+    (void)display;
     /* dummy implementation, no need to do anything */
 }
 
@@ -104,11 +88,8 @@ TkCreateXEventSource(void)
 #   define TkUnixContainerId 0
 #   define TkUnixDoOneXEvent 0
 #   define TkUnixSetMenubar 0
-#   define XCreateWindow 0
-#   define XOffsetRegion 0
-#   define XUnionRegion 0
-#   define TkWmCleanup (void (*)(TkDisplay *)) TkpSync
-#   define TkSendCleanup (void (*)(TkDisplay *)) TkpSync
+#   define TkWmCleanup (void (*)(TkDisplay *))(void *)TkpSync
+#   define TkSendCleanup (void (*)(TkDisplay *))(void *)TkpSync
 #   define TkpTestsendCmd 0
 
 #else /* !_WIN32 */
@@ -127,22 +108,21 @@ TkCreateXEventSource(void)
  */
 
 #define GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS	0x00000004
+#ifdef __cplusplus
+extern "C" {
+#endif
 int __stdcall GetModuleHandleExW(unsigned int, const char *, void *);
-
-void *Tk_GetHINSTANCE()
-{
-    void *hInstance = NULL;
-
-    GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
-	    (const char *) &tkIntStubs, &hInstance);
-    return hInstance;
+#ifdef __cplusplus
 }
+#endif
 
 void
 TkSetPixmapColormap(
     Pixmap pixmap,
     Colormap colormap)
 {
+    (void)pixmap;
+    (void)colormap;
 }
 
 void
@@ -168,55 +148,16 @@ TkPutImage(
     unsigned int width, unsigned int height)
 				/* Dimensions of subimage. */
 {
+    (void)colors;
+    (void)ncolors;
+
     return XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
 }
 
-TkRegion TkCreateRegion()
-{
-    return (TkRegion) XCreateRegion();
-}
-
-int TkDestroyRegion(TkRegion r)
-{
-	return XDestroyRegion((Region)r);
-}
-
-int TkSetRegion(Display *d, GC g, TkRegion r)
-{
-	return XSetRegion(d, g, (Region)r);
-}
-
-int TkUnionRectWithRegion(XRectangle *a, TkRegion b, TkRegion c)
-{
-	return XUnionRectWithRegion(a, (Region) b, (Region) c);
-}
-
-int TkClipBox(TkRegion a, XRectangle *b)
-{
-	return XClipBox((Region) a, b);
-}
-
-int TkIntersectRegion(TkRegion a, TkRegion b, TkRegion c)
-{
-	return XIntersectRegion((Region) a, (Region) b, (Region) c);
-}
-
-int TkRectInRegion (TkRegion r, int a, int b, unsigned int c, unsigned int d)
-{
-    return XRectInRegion((Region) r, a, b, c, d);
-}
-
-int TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
-{
-    return XSubtractRegion((Region) a, (Region) b, (Region) c);
-}
-
 	/* TODO: To be implemented for Cygwin */
 #	define Tk_AttachHWND 0
 #	define Tk_GetHWND 0
 #	define Tk_HWNDToWindow 0
-#	define Tk_PointerEvent 0
-#	define Tk_TranslateWinEvent 0
 #	define TkAlignImageData 0
 #	define TkpGetMS 0
 #	define TkpGetCapture 0
@@ -245,33 +186,37 @@ int TkSubtractRegion (TkRegion a, TkRegion b, TkRegion c)
 #	define TkWinSetForegroundWindow 0
 #	define TkWinDialogDebug 0
 #	define TkWinGetMenuSystemDefault 0
-#	define TkWinGetPlatformId 0
 #	define TkWinSetHINSTANCE 0
 #	define TkWinGetPlatformTheme 0
 #	define TkWinChildProc 0
 
-#   elif !defined(MAC_OSX_TK) /* UNIX */
-
-#	undef TkClipBox
-#	undef TkCreateRegion
-#	undef TkDestroyRegion
-#	undef TkIntersectRegion
-#	undef TkRectInRegion
-#	undef TkSetRegion
-#	undef TkUnionRectWithRegion
-#	undef TkSubtractRegion
-
-#	define TkClipBox (int (*) (TkRegion, XRectangle *)) XClipBox
-#	define TkCreateRegion (TkRegion (*) ()) XCreateRegion
-#	define TkDestroyRegion (int (*) (TkRegion)) XDestroyRegion
-#	define TkIntersectRegion (int (*) (TkRegion, TkRegion, TkRegion)) XIntersectRegion
-#	define TkRectInRegion (int (*) (TkRegion, int, int, unsigned int, unsigned int)) XRectInRegion
-#	define TkSetRegion (int (*) (Display *, GC, TkRegion)) XSetRegion
-#	define TkUnionRectWithRegion (int (*) (XRectangle *, TkRegion, TkRegion)) XUnionRectWithRegion
-#	define TkSubtractRegion (int (*) (TkRegion, TkRegion, TkRegion)) XSubtractRegion
 #   endif
 #endif /* !_WIN32 */
 
+#if defined(MAC_OSX_TCL)
+
+int
+TkPutImage(
+    unsigned long *colors,	/* Array of pixel values used by this image.
+				 * May be NULL. */
+    int ncolors,		/* Number of colors used, or 0. */
+    Display *display,
+    Drawable d,			/* Destination drawable. */
+    GC gc,
+    XImage *image,		/* Source image. */
+    int src_x, int src_y,	/* Offset of subimage. */
+    int dest_x, int dest_y,	/* Position of subimage origin in drawable. */
+    unsigned int width, unsigned int height)
+				/* Dimensions of subimage. */
+{
+	(void)colors;
+	(void)ncolors;
+
+    return XPutImage(display, d, gc, image, src_x, src_y, dest_x, dest_y, width, height);
+}
+#endif /* MAC_OSX_TCL */
+
+
 /*
  * WARNING: The contents of this file is automatically generated by the
  * tools/genStubs.tcl script. Any modifications to the function declarations
@@ -351,13 +296,13 @@ static const TkIntStubs tkIntStubs = {
     TkpClaimFocus, /* 57 */
     TkpDisplayWarning, /* 58 */
     TkpGetAppName, /* 59 */
-    TkpGetOtherWindow, /* 60 */
+    0, /* 60 */
     TkpGetWrapperWindow, /* 61 */
     TkpInit, /* 62 */
     TkpInitializeMenuBindings, /* 63 */
-    TkpMakeContainer, /* 64 */
+    0, /* 64 */
     TkpMakeMenuWindow, /* 65 */
-    TkpMakeWindow, /* 66 */
+    0, /* 66 */
     TkpMenuNotifyToplevelCreate, /* 67 */
     TkpOpenDisplay, /* 68 */
     TkPointerEvent, /* 69 */
@@ -365,8 +310,8 @@ static const TkIntStubs tkIntStubs = {
     TkPolygonToPoint, /* 71 */
     TkPositionInTree, /* 72 */
     TkpRedirectKeyEvent, /* 73 */
-    TkpSetMainMenubar, /* 74 */
-    TkpUseWindow, /* 75 */
+    0, /* 74 */
+    0, /* 75 */
     0, /* 76 */
     TkQueueEventForAllChildren, /* 77 */
     TkReadBitmapFile, /* 78 */
@@ -376,7 +321,7 @@ static const TkIntStubs tkIntStubs = {
     TkSelInit, /* 82 */
     TkSelPropProc, /* 83 */
     0, /* 84 */
-    TkSetWindowMenuBar, /* 85 */
+    0, /* 85 */
     TkStringToKeysym, /* 86 */
     TkThickPolyLineToArea, /* 87 */
     TkWmAddToColormapWindows, /* 88 */
@@ -402,53 +347,20 @@ static const TkIntStubs tkIntStubs = {
     TkGetWindowFromObj, /* 108 */
     TkpGetString, /* 109 */
     TkpGetSubFonts, /* 110 */
-    TkpGetSystemDefault, /* 111 */
+    0, /* 111 */
     TkpMenuThreadInit, /* 112 */
-    TkClipBox, /* 113 */
-    TkCreateRegion, /* 114 */
-    TkDestroyRegion, /* 115 */
-    TkIntersectRegion, /* 116 */
-    TkRectInRegion, /* 117 */
-    TkSetRegion, /* 118 */
-    TkUnionRectWithRegion, /* 119 */
+    XClipBox, /* 113 */
+    XCreateRegion, /* 114 */
+    XDestroyRegion, /* 115 */
+    XIntersectRegion, /* 116 */
+    XRectInRegion, /* 117 */
+    XSetRegion, /* 118 */
+    XUnionRectWithRegion, /* 119 */
     0, /* 120 */
-#if !(defined(_WIN32) || defined(MAC_OSX_TK)) /* X11 */
-    0, /* 121 */
-#endif /* X11 */
-#if defined(_WIN32) /* WIN */
-    0, /* 121 */
-#endif /* WIN */
-#ifdef MAC_OSX_TK /* AQUA */
-# if TCL_MAJOR_VERSION < 9
-    0, /* 121 */ /* Dummy entry for stubs table backwards compatibility */
-# endif /* TCL_MAJOR_VERSION < 9 */
     TkpCreateNativeBitmap, /* 121 */
-#endif /* AQUA */
-#if !(defined(_WIN32) || defined(MAC_OSX_TK)) /* X11 */
-    0, /* 122 */
-#endif /* X11 */
-#if defined(_WIN32) /* WIN */
-    0, /* 122 */
-#endif /* WIN */
-#ifdef MAC_OSX_TK /* AQUA */
-# if TCL_MAJOR_VERSION < 9
-    0, /* 122 */ /* Dummy entry for stubs table backwards compatibility */
-# endif /* TCL_MAJOR_VERSION < 9 */
     TkpDefineNativeBitmaps, /* 122 */
-#endif /* AQUA */
     0, /* 123 */
-#if !(defined(_WIN32) || defined(MAC_OSX_TK)) /* X11 */
-    0, /* 124 */
-#endif /* X11 */
-#if defined(_WIN32) /* WIN */
-    0, /* 124 */
-#endif /* WIN */
-#ifdef MAC_OSX_TK /* AQUA */
-# if TCL_MAJOR_VERSION < 9
-    0, /* 124 */ /* Dummy entry for stubs table backwards compatibility */
-# endif /* TCL_MAJOR_VERSION < 9 */
     TkpGetNativeAppBitmap, /* 124 */
-#endif /* AQUA */
     0, /* 125 */
     0, /* 126 */
     0, /* 127 */
@@ -459,7 +371,7 @@ static const TkIntStubs tkIntStubs = {
     0, /* 132 */
     0, /* 133 */
     0, /* 134 */
-    TkpDrawHighlightBorder, /* 135 */
+    0, /* 135 */
     TkSetFocusWin, /* 136 */
     TkpSetKeycodeAndState, /* 137 */
     TkpGetKeySym, /* 138 */
@@ -469,7 +381,7 @@ static const TkIntStubs tkIntStubs = {
     TkFocusFree, /* 142 */
     TkClipCleanup, /* 143 */
     TkGCCleanup, /* 144 */
-    TkSubtractRegion, /* 145 */
+    XSubtractRegion, /* 145 */
     TkStylePkgInit, /* 146 */
     TkStylePkgFree, /* 147 */
     TkToplevelWindowForCommand, /* 148 */
@@ -509,32 +421,16 @@ static const TkIntStubs tkIntStubs = {
     TkUnderlineAngledTextLayout, /* 182 */
     TkIntersectAngledTextLayout, /* 183 */
     TkDrawAngledChars, /* 184 */
-#if !defined(_WIN32) && !defined(MAC_OSX_TCL) /* UNIX */
-    0, /* 185 */
-#endif /* UNIX */
-#if defined(_WIN32) /* WIN */
-    0, /* 185 */
-#endif /* WIN */
-#ifdef MAC_OSX_TCL /* MACOSX */
     TkpRedrawWidget, /* 185 */
-#endif /* MACOSX */
-#if !defined(_WIN32) && !defined(MAC_OSX_TCL) /* UNIX */
-    0, /* 186 */
-#endif /* UNIX */
-#if defined(_WIN32) /* WIN */
-    0, /* 186 */
-#endif /* WIN */
-#ifdef MAC_OSX_TCL /* MACOSX */
     TkpWillDrawWidget, /* 186 */
-#endif /* MACOSX */
-    TkUnusedStubEntry, /* 187 */
+    TkDebugPhotoStringMatchDef, /* 187 */
 };
 
 static const TkIntPlatStubs tkIntPlatStubs = {
     TCL_STUB_MAGIC,
     0,
 #if defined(_WIN32) || defined(__CYGWIN__) /* WIN */
-    TkAlignImageData, /* 0 */
+    TkCreateXEventSource, /* 0 */
     0, /* 1 */
     TkGenerateActivateEvents, /* 2 */
     TkpGetMS, /* 3 */
@@ -567,11 +463,11 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     TkWinSetForegroundWindow, /* 30 */
     TkWinDialogDebug, /* 31 */
     TkWinGetMenuSystemDefault, /* 32 */
-    TkWinGetPlatformId, /* 33 */
+    TkAlignImageData, /* 33 */
     TkWinSetHINSTANCE, /* 34 */
     TkWinGetPlatformTheme, /* 35 */
     TkWinChildProc, /* 36 */
-    TkCreateXEventSource, /* 37 */
+    0, /* 37 */
     TkpCmapStressed, /* 38 */
     TkpSync, /* 39 */
     TkUnixContainerId, /* 40 */
@@ -584,18 +480,18 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     TkpGetCapture, /* 47 */
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
-    TkGenerateActivateEvents, /* 0 */
-    0, /* 1 */
-    TkGenerateActivateEvents_, /* 2 */
-    TkPointerDeadWindow, /* 3 */
-    TkpSetCapture, /* 4 */
+    0, /* 0 */
+    TkAboutDlg, /* 1 */
+    TkGenerateActivateEvents, /* 2 */
+    TkpGetMS, /* 3 */
+    TkPointerDeadWindow, /* 4 */
     TkpSetCursor, /* 5 */
-    TkpWmSetState, /* 6 */
-    TkAboutDlg, /* 7 */
+    TkpScanWindowId, /* 6 */
+    TkpWmSetState, /* 7 */
     TkMacOSXButtonKeyState, /* 8 */
     TkMacOSXClearMenubarActive, /* 9 */
     TkMacOSXDispatchMenuEvent, /* 10 */
-    TkMacOSXInstallCursor, /* 11 */
+    TkpSetCapture, /* 11 */
     TkMacOSXHandleTearoffMenu, /* 12 */
     0, /* 13 */
     TkMacOSXDoHLEvent, /* 14 */
@@ -606,7 +502,7 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     0, /* 19 */
     0, /* 20 */
     TkMacOSXInvalidateWindow, /* 21 */
-    TkMacOSXIsCharacterMissing, /* 22 */
+    0, /* 22 */
     TkMacOSXMakeRealWindowExist, /* 23 */
     TkMacOSXMakeStippleMap, /* 24 */
     TkMacOSXMenuClick, /* 25 */
@@ -614,7 +510,7 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     TkMacOSXResizable, /* 27 */
     TkMacOSXSetHelpMenuItemCount, /* 28 */
     TkMacOSXSetScrollbarGrow, /* 29 */
-    TkMacOSXSetUpClippingRgn, /* 30 */
+    0, /* 30 */
     TkMacOSXSetUpGraphicsPort, /* 31 */
     TkMacOSXUpdateClipRgn, /* 32 */
     0, /* 33 */
@@ -631,31 +527,27 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     TkMacOSXGetHostToplevel, /* 44 */
     TkMacOSXPreprocessMenu, /* 45 */
     TkpIsWindowFloating, /* 46 */
-    TkMacOSXGetCapture, /* 47 */
+    TkpGetCapture, /* 47 */
     0, /* 48 */
-    TkGetTransientMaster, /* 49 */
+    TkMacOSXGetContainer, /* 49 */
     TkGenerateButtonEvent, /* 50 */
     TkGenWMDestroyEvent, /* 51 */
-    TkMacOSXSetDrawingEnabled, /* 52 */
-    TkpGetMS, /* 53 */
-    TkMacOSXDrawable, /* 54 */
-    TkpScanWindowId, /* 55 */
 #endif /* AQUA */
 #if !(defined(_WIN32) || defined(__CYGWIN__) || defined(MAC_OSX_TK)) /* X11 */
     TkCreateXEventSource, /* 0 */
     0, /* 1 */
     TkGenerateActivateEvents, /* 2 */
-    TkpCmapStressed, /* 3 */
-    TkpSync, /* 4 */
-    TkUnixContainerId, /* 5 */
-    TkUnixDoOneXEvent, /* 6 */
-    TkUnixSetMenubar, /* 7 */
-    TkpScanWindowId, /* 8 */
-    TkWmCleanup, /* 9 */
-    TkSendCleanup, /* 10 */
+    0, /* 3 */
+    0, /* 4 */
+    0, /* 5 */
+    TkpScanWindowId, /* 6 */
+    0, /* 7 */
+    0, /* 8 */
+    TkpWmSetState, /* 9 */
+    0, /* 10 */
     0, /* 11 */
-    TkpWmSetState, /* 12 */
-    TkpTestsendCmd, /* 13 */
+    0, /* 12 */
+    0, /* 13 */
     0, /* 14 */
     0, /* 15 */
     0, /* 16 */
@@ -680,14 +572,14 @@ static const TkIntPlatStubs tkIntPlatStubs = {
     0, /* 35 */
     0, /* 36 */
     0, /* 37 */
-    TkpCmapStressed_, /* 38 */
-    TkpSync_, /* 39 */
-    TkUnixContainerId_, /* 40 */
-    TkUnixDoOneXEvent_, /* 41 */
-    TkUnixSetMenubar_, /* 42 */
-    TkWmCleanup_, /* 43 */
-    TkSendCleanup_, /* 44 */
-    TkpTestsendCmd_, /* 45 */
+    TkpCmapStressed, /* 38 */
+    TkpSync, /* 39 */
+    TkUnixContainerId, /* 40 */
+    TkUnixDoOneXEvent, /* 41 */
+    TkUnixSetMenubar, /* 42 */
+    TkWmCleanup, /* 43 */
+    TkSendCleanup, /* 44 */
+    TkpTestsendCmd, /* 45 */
 #endif /* X11 */
 };
 
@@ -833,29 +725,29 @@ static const TkIntXlibStubs tkIntXlibStubs = {
     XDrawPoints, /* 135 */
     XReparentWindow, /* 136 */
     XPutImage, /* 137 */
-    0, /* 138 */
-    0, /* 139 */
-    0, /* 140 */
-    0, /* 141 */
-    0, /* 142 */
-    0, /* 143 */
+    XPolygonRegion, /* 138 */
+    XPointInRegion, /* 139 */
+    XVaCreateNestedList, /* 140 */
+    XSetICValues, /* 141 */
+    XGetICValues, /* 142 */
+    XSetICFocus, /* 143 */
     0, /* 144 */
     0, /* 145 */
     0, /* 146 */
-    0, /* 147 */
-    0, /* 148 */
-    0, /* 149 */
-    0, /* 150 */
-    0, /* 151 */
-    0, /* 152 */
-    0, /* 153 */
-    0, /* 154 */
-    0, /* 155 */
-    0, /* 156 */
-    0, /* 157 */
-    TkUnusedStubEntry, /* 158 */
+    XFreeFontSet, /* 147 */
+    XCloseIM, /* 148 */
+    XRegisterIMInstantiateCallback, /* 149 */
+    XUnregisterIMInstantiateCallback, /* 150 */
+    XSetLocaleModifiers, /* 151 */
+    XOpenIM, /* 152 */
+    XGetIMValues, /* 153 */
+    XSetIMValues, /* 154 */
+    XCreateFontSet, /* 155 */
+    XFreeStringList, /* 156 */
+    XkbKeycodeToKeysym, /* 157 */
+    XkbOpenDisplay, /* 158 */
 #endif /* WIN */
-#ifdef MAC_OSX_TK /* AQUA */
+#ifdef MAC_OSX_TCL /* MACOSX */
     XSetDashes, /* 0 */
     XGetModifierMapping, /* 1 */
     XCreateImage, /* 2 */
@@ -863,113 +755,113 @@ static const TkIntXlibStubs tkIntXlibStubs = {
     XGetAtomName, /* 4 */
     XKeysymToString, /* 5 */
     XCreateColormap, /* 6 */
-    XGContextFromGC, /* 7 */
-    XKeycodeToKeysym, /* 8 */
-    XStringToKeysym, /* 9 */
-    XRootWindow, /* 10 */
-    XSetErrorHandler, /* 11 */
-    XAllocColor, /* 12 */
-    XBell, /* 13 */
-    XChangeProperty, /* 14 */
-    XChangeWindowAttributes, /* 15 */
-    XConfigureWindow, /* 16 */
-    XCopyArea, /* 17 */
-    XCopyPlane, /* 18 */
-    XCreateBitmapFromData, /* 19 */
-    XDefineCursor, /* 20 */
-    XDestroyWindow, /* 21 */
-    XDrawArc, /* 22 */
-    XDrawLines, /* 23 */
-    XDrawRectangle, /* 24 */
-    XFillArc, /* 25 */
-    XFillPolygon, /* 26 */
-    XFillRectangles, /* 27 */
-    XFreeColormap, /* 28 */
-    XFreeColors, /* 29 */
-    XFreeModifiermap, /* 30 */
-    XGetGeometry, /* 31 */
-    XGetWindowProperty, /* 32 */
-    XGrabKeyboard, /* 33 */
-    XGrabPointer, /* 34 */
-    XKeysymToKeycode, /* 35 */
-    XMapWindow, /* 36 */
-    XMoveResizeWindow, /* 37 */
-    XMoveWindow, /* 38 */
-    XQueryPointer, /* 39 */
-    XRaiseWindow, /* 40 */
-    XRefreshKeyboardMapping, /* 41 */
-    XResizeWindow, /* 42 */
-    XSelectInput, /* 43 */
-    XSendEvent, /* 44 */
-    XSetIconName, /* 45 */
-    XSetInputFocus, /* 46 */
-    XSetSelectionOwner, /* 47 */
-    XSetWindowBackground, /* 48 */
-    XSetWindowBackgroundPixmap, /* 49 */
-    XSetWindowBorder, /* 50 */
-    XSetWindowBorderPixmap, /* 51 */
-    XSetWindowBorderWidth, /* 52 */
-    XSetWindowColormap, /* 53 */
-    XUngrabKeyboard, /* 54 */
-    XUngrabPointer, /* 55 */
-    XUnmapWindow, /* 56 */
-    TkPutImage, /* 57 */
-    XParseColor, /* 58 */
-    XCreateGC, /* 59 */
-    XFreeGC, /* 60 */
-    XInternAtom, /* 61 */
-    XSetBackground, /* 62 */
-    XSetForeground, /* 63 */
-    XSetClipMask, /* 64 */
-    XSetClipOrigin, /* 65 */
-    XSetTSOrigin, /* 66 */
-    XChangeGC, /* 67 */
-    XSetFont, /* 68 */
-    XSetArcMode, /* 69 */
-    XSetStipple, /* 70 */
-    XSetFillRule, /* 71 */
-    XSetFillStyle, /* 72 */
-    XSetFunction, /* 73 */
-    XSetLineAttributes, /* 74 */
-    _XInitImageFuncPtrs, /* 75 */
-    XCreateIC, /* 76 */
-    XGetVisualInfo, /* 77 */
-    XSetWMClientMachine, /* 78 */
-    XStringListToTextProperty, /* 79 */
-    XDrawSegments, /* 80 */
-    XForceScreenSaver, /* 81 */
-    XDrawLine, /* 82 */
-    XFillRectangle, /* 83 */
-    XClearWindow, /* 84 */
-    XDrawPoint, /* 85 */
-    XDrawPoints, /* 86 */
-    XWarpPointer, /* 87 */
-    XQueryColor, /* 88 */
-    XQueryColors, /* 89 */
-    XQueryTree, /* 90 */
-    XSync, /* 91 */
-    0, /* 92 */
-    0, /* 93 */
-    0, /* 94 */
-    0, /* 95 */
-    0, /* 96 */
-    0, /* 97 */
-    0, /* 98 */
-    0, /* 99 */
-    0, /* 100 */
-    0, /* 101 */
-    0, /* 102 */
-    0, /* 103 */
-    0, /* 104 */
-    0, /* 105 */
-    XSetClipRectangles, /* 106 */
+    XCreatePixmapCursor, /* 7 */
+    XCreateGlyphCursor, /* 8 */
+    XGContextFromGC, /* 9 */
+    XListHosts, /* 10 */
+    XKeycodeToKeysym, /* 11 */
+    XStringToKeysym, /* 12 */
+    XRootWindow, /* 13 */
+    XSetErrorHandler, /* 14 */
+    XIconifyWindow, /* 15 */
+    XWithdrawWindow, /* 16 */
+    XGetWMColormapWindows, /* 17 */
+    XAllocColor, /* 18 */
+    XBell, /* 19 */
+    XChangeProperty, /* 20 */
+    XChangeWindowAttributes, /* 21 */
+    XClearWindow, /* 22 */
+    XConfigureWindow, /* 23 */
+    XCopyArea, /* 24 */
+    XCopyPlane, /* 25 */
+    XCreateBitmapFromData, /* 26 */
+    XDefineCursor, /* 27 */
+    XDeleteProperty, /* 28 */
+    XDestroyWindow, /* 29 */
+    XDrawArc, /* 30 */
+    XDrawLines, /* 31 */
+    XDrawRectangle, /* 32 */
+    XFillArc, /* 33 */
+    XFillPolygon, /* 34 */
+    XFillRectangles, /* 35 */
+    XForceScreenSaver, /* 36 */
+    XFreeColormap, /* 37 */
+    XFreeColors, /* 38 */
+    XFreeCursor, /* 39 */
+    XFreeModifiermap, /* 40 */
+    XGetGeometry, /* 41 */
+    XGetInputFocus, /* 42 */
+    XGetWindowProperty, /* 43 */
+    XGetWindowAttributes, /* 44 */
+    XGrabKeyboard, /* 45 */
+    XGrabPointer, /* 46 */
+    XKeysymToKeycode, /* 47 */
+    XLookupColor, /* 48 */
+    XMapWindow, /* 49 */
+    XMoveResizeWindow, /* 50 */
+    XMoveWindow, /* 51 */
+    XNextEvent, /* 52 */
+    XPutBackEvent, /* 53 */
+    XQueryColors, /* 54 */
+    XQueryPointer, /* 55 */
+    XQueryTree, /* 56 */
+    XRaiseWindow, /* 57 */
+    XRefreshKeyboardMapping, /* 58 */
+    XResizeWindow, /* 59 */
+    XSelectInput, /* 60 */
+    XSendEvent, /* 61 */
+    XSetCommand, /* 62 */
+    XSetIconName, /* 63 */
+    XSetInputFocus, /* 64 */
+    XSetSelectionOwner, /* 65 */
+    XSetWindowBackground, /* 66 */
+    XSetWindowBackgroundPixmap, /* 67 */
+    XSetWindowBorder, /* 68 */
+    XSetWindowBorderPixmap, /* 69 */
+    XSetWindowBorderWidth, /* 70 */
+    XSetWindowColormap, /* 71 */
+    XTranslateCoordinates, /* 72 */
+    XUngrabKeyboard, /* 73 */
+    XUngrabPointer, /* 74 */
+    XUnmapWindow, /* 75 */
+    XWindowEvent, /* 76 */
+    XDestroyIC, /* 77 */
+    XFilterEvent, /* 78 */
+    XmbLookupString, /* 79 */
+    TkPutImage, /* 80 */
+    XSetClipRectangles, /* 81 */
+    XParseColor, /* 82 */
+    XCreateGC, /* 83 */
+    XFreeGC, /* 84 */
+    XInternAtom, /* 85 */
+    XSetBackground, /* 86 */
+    XSetForeground, /* 87 */
+    XSetClipMask, /* 88 */
+    XSetClipOrigin, /* 89 */
+    XSetTSOrigin, /* 90 */
+    XChangeGC, /* 91 */
+    XSetFont, /* 92 */
+    XSetArcMode, /* 93 */
+    XSetStipple, /* 94 */
+    XSetFillRule, /* 95 */
+    XSetFillStyle, /* 96 */
+    XSetFunction, /* 97 */
+    XSetLineAttributes, /* 98 */
+    _XInitImageFuncPtrs, /* 99 */
+    XCreateIC, /* 100 */
+    XGetVisualInfo, /* 101 */
+    XSetWMClientMachine, /* 102 */
+    XStringListToTextProperty, /* 103 */
+    XDrawLine, /* 104 */
+    XWarpPointer, /* 105 */
+    XFillRectangle, /* 106 */
     XFlush, /* 107 */
     XGrabServer, /* 108 */
     XUngrabServer, /* 109 */
     XFree, /* 110 */
     XNoOp, /* 111 */
     XSynchronize, /* 112 */
-    0, /* 113 */
+    XSync, /* 113 */
     XVisualIDFromVisual, /* 114 */
     0, /* 115 */
     0, /* 116 */
@@ -977,8 +869,8 @@ static const TkIntXlibStubs tkIntXlibStubs = {
     0, /* 118 */
     0, /* 119 */
     XOffsetRegion, /* 120 */
-    0, /* 121 */
-    0, /* 122 */
+    XUnionRegion, /* 121 */
+    XCreateWindow, /* 122 */
     0, /* 123 */
     0, /* 124 */
     0, /* 125 */
@@ -986,36 +878,36 @@ static const TkIntXlibStubs tkIntXlibStubs = {
     0, /* 127 */
     0, /* 128 */
     XLowerWindow, /* 129 */
-    0, /* 130 */
-    0, /* 131 */
-    0, /* 132 */
-    0, /* 133 */
-    0, /* 134 */
-    0, /* 135 */
-    0, /* 136 */
+    XFillArcs, /* 130 */
+    XDrawArcs, /* 131 */
+    XDrawRectangles, /* 132 */
+    XDrawSegments, /* 133 */
+    XDrawPoint, /* 134 */
+    XDrawPoints, /* 135 */
+    XReparentWindow, /* 136 */
     XPutImage, /* 137 */
-    0, /* 138 */
-    0, /* 139 */
-    0, /* 140 */
-    0, /* 141 */
-    0, /* 142 */
-    0, /* 143 */
-    XDestroyIC, /* 144 */
-    XCreatePixmapCursor, /* 145 */
-    XCreateGlyphCursor, /* 146 */
-    0, /* 147 */
-    0, /* 148 */
-    0, /* 149 */
-    0, /* 150 */
-    0, /* 151 */
-    0, /* 152 */
-    0, /* 153 */
-    0, /* 154 */
-    0, /* 155 */
-    0, /* 156 */
+    XPolygonRegion, /* 138 */
+    XPointInRegion, /* 139 */
+    XVaCreateNestedList, /* 140 */
+    XSetICValues, /* 141 */
+    XGetICValues, /* 142 */
+    XSetICFocus, /* 143 */
+    0, /* 144 */
+    0, /* 145 */
+    0, /* 146 */
+    XFreeFontSet, /* 147 */
+    XCloseIM, /* 148 */
+    XRegisterIMInstantiateCallback, /* 149 */
+    XUnregisterIMInstantiateCallback, /* 150 */
+    XSetLocaleModifiers, /* 151 */
+    XOpenIM, /* 152 */
+    XGetIMValues, /* 153 */
+    XSetIMValues, /* 154 */
+    XCreateFontSet, /* 155 */
+    XFreeStringList, /* 156 */
     XkbKeycodeToKeysym, /* 157 */
-    TkUnusedStubEntry, /* 158 */
-#endif /* AQUA */
+    XkbOpenDisplay, /* 158 */
+#endif /* MACOSX */
 };
 
 static const TkPlatStubs tkPlatStubs = {
@@ -1026,18 +918,16 @@ static const TkPlatStubs tkPlatStubs = {
     Tk_GetHINSTANCE, /* 1 */
     Tk_GetHWND, /* 2 */
     Tk_HWNDToWindow, /* 3 */
-    Tk_PointerEvent, /* 4 */
-    Tk_TranslateWinEvent, /* 5 */
 #endif /* WIN */
 #ifdef MAC_OSX_TK /* AQUA */
-    Tk_MacOSXSetEmbedHandler, /* 0 */
-    Tk_MacOSXTurnOffMenus, /* 1 */
-    Tk_MacOSXTkOwnsCursor, /* 2 */
-    TkMacOSXInitMenus, /* 3 */
+    0, /* 0 */
+    0, /* 1 */
+    0, /* 2 */
+    0, /* 3 */
     TkMacOSXInitAppleEvents, /* 4 */
-    TkGenWMConfigureEvent, /* 5 */
+    0, /* 5 */
     TkMacOSXInvalClipRgns, /* 6 */
-    TkMacOSXGetDrawablePort, /* 7 */
+    0, /* 7 */
     TkMacOSXGetRootControl, /* 8 */
     Tk_MacOSXSetupTkNotifier, /* 9 */
     Tk_MacOSXIsAppInFront, /* 10 */
@@ -1046,7 +936,7 @@ static const TkPlatStubs tkPlatStubs = {
     Tk_MacOSXGetNSWindowForDrawable, /* 13 */
     0, /* 14 */
     0, /* 15 */
-    TkGenWMConfigureEvent_, /* 16 */
+    TkGenWMConfigureEvent, /* 16 */
 #endif /* AQUA */
 };
 
@@ -1137,7 +1027,7 @@ const TkStubs tkStubs = {
     Tk_FreeOptions, /* 74 */
     Tk_FreePixmap, /* 75 */
     Tk_FreeTextLayout, /* 76 */
-    Tk_FreeXId, /* 77 */
+    0, /* 77 */
     Tk_GCForColor, /* 78 */
     Tk_GeometryRequest, /* 79 */
     Tk_Get3DBorder, /* 80 */
@@ -1158,7 +1048,7 @@ const TkStubs tkStubs = {
     Tk_GetFontMetrics, /* 95 */
     Tk_GetGC, /* 96 */
     Tk_GetImage, /* 97 */
-    Tk_GetImageMasterData, /* 98 */
+    Tk_GetImageModelData, /* 98 */
     Tk_GetItemTypes, /* 99 */
     Tk_GetJoinStyle, /* 100 */
     Tk_GetJustify, /* 101 */
@@ -1178,7 +1068,7 @@ const TkStubs tkStubs = {
     Tk_HandleEvent, /* 115 */
     Tk_IdToWindow, /* 116 */
     Tk_ImageChanged, /* 117 */
-    Tk_Init, /* 118 */
+    0, /* 118 */
     Tk_InternAtom, /* 119 */
     Tk_IntersectTextLayout, /* 120 */
     Tk_MaintainGeometry, /* 121 */
@@ -1204,13 +1094,13 @@ const TkStubs tkStubs = {
     Tk_NameToWindow, /* 141 */
     Tk_OwnSelection, /* 142 */
     Tk_ParseArgv, /* 143 */
-    Tk_PhotoPutBlock_NoComposite, /* 144 */
-    Tk_PhotoPutZoomedBlock_NoComposite, /* 145 */
+    0, /* 144 */
+    0, /* 145 */
     Tk_PhotoGetImage, /* 146 */
     Tk_PhotoBlank, /* 147 */
-    Tk_PhotoExpand_Panic, /* 148 */
+    0, /* 148 */
     Tk_PhotoGetSize, /* 149 */
-    Tk_PhotoSetSize_Panic, /* 150 */
+    0, /* 150 */
     Tk_PointToChar, /* 151 */
     Tk_PostscriptFontName, /* 152 */
     Tk_PreserveColormap, /* 153 */
@@ -1219,7 +1109,7 @@ const TkStubs tkStubs = {
     Tk_ResizeWindow, /* 156 */
     Tk_RestackWindow, /* 157 */
     Tk_RestrictEvents, /* 158 */
-    Tk_SafeInit, /* 159 */
+    0, /* 159 */
     Tk_SetAppName, /* 160 */
     Tk_SetBackgroundFromBorder, /* 161 */
     Tk_SetClass, /* 162 */
@@ -1272,11 +1162,11 @@ const TkStubs tkStubs = {
     Tk_GetReliefFromObj, /* 209 */
     Tk_GetScrollInfoObj, /* 210 */
     Tk_InitOptions, /* 211 */
-    Tk_MainEx, /* 212 */
+    0, /* 212 */
     Tk_RestoreSavedOptions, /* 213 */
     Tk_SetOptions, /* 214 */
     Tk_InitConsoleChannels, /* 215 */
-    Tk_CreateConsoleWindow, /* 216 */
+    0, /* 216 */
     Tk_CreateSmoothMethod, /* 217 */
     0, /* 218 */
     0, /* 219 */
@@ -1306,8 +1196,8 @@ const TkStubs tkStubs = {
     Tk_SetInternalBorderEx, /* 243 */
     Tk_SetMinimumRequestSize, /* 244 */
     Tk_SetCaretPos, /* 245 */
-    Tk_PhotoPutBlock_Panic, /* 246 */
-    Tk_PhotoPutZoomedBlock_Panic, /* 247 */
+    0, /* 246 */
+    0, /* 247 */
     Tk_CollapseMotionEvents, /* 248 */
     Tk_RegisterStyleEngine, /* 249 */
     Tk_GetStyleEngine, /* 250 */
@@ -1318,8 +1208,8 @@ const TkStubs tkStubs = {
     Tk_FreeStyle, /* 255 */
     Tk_NameOfStyle, /* 256 */
     Tk_AllocStyleFromObj, /* 257 */
-    Tk_GetStyleFromObj, /* 258 */
-    Tk_FreeStyleFromObj, /* 259 */
+    0, /* 258 */
+    0, /* 259 */
     Tk_GetStyledElement, /* 260 */
     Tk_GetElementSize, /* 261 */
     Tk_GetElementBox, /* 262 */
@@ -1332,25 +1222,38 @@ const TkStubs tkStubs = {
     Tk_GetUserInactiveTime, /* 269 */
     Tk_ResetUserInactiveTime, /* 270 */
     Tk_Interp, /* 271 */
-    Tk_CreateOldImageType, /* 272 */
-    Tk_CreateOldPhotoImageFormat, /* 273 */
-    0, /* 274 */
-    0, /* 275 */
-    0, /* 276 */
-    0, /* 277 */
-    0, /* 278 */
-    0, /* 279 */
-    0, /* 280 */
-    0, /* 281 */
-    0, /* 282 */
-    0, /* 283 */
-    0, /* 284 */
-    0, /* 285 */
-    0, /* 286 */
-    0, /* 287 */
-    0, /* 288 */
-    0, /* 289 */
-    TkUnusedStubEntry, /* 290 */
+    0, /* 272 */
+    0, /* 273 */
+    Tk_AlwaysShowSelection, /* 274 */
+    Tk_GetButtonMask, /* 275 */
+    Tk_GetDoublePixelsFromObj, /* 276 */
+    Tk_NewWindowObj, /* 277 */
+    Tk_SendVirtualEvent, /* 278 */
+    Tk_FontGetDescription, /* 279 */
+    Tk_CreatePhotoImageFormatVersion3, /* 280 */
+    Tk_DrawHighlightBorder, /* 281 */
+    Tk_SetMainMenubar, /* 282 */
+    Tk_SetWindowMenubar, /* 283 */
+    Tk_ClipDrawableToRect, /* 284 */
+    Tk_GetSystemDefault, /* 285 */
+    Tk_UseWindow, /* 286 */
+    Tk_MakeContainer, /* 287 */
+    Tk_GetOtherWindow, /* 288 */
+    Tk_Get3DBorderColors, /* 289 */
+    Tk_MakeWindow, /* 290 */
 };
 
 /* !END!: Do not edit above this line. */
+
+
+#ifdef __CYGWIN__
+void *Tk_GetHINSTANCE(void)
+{
+    void *hInstance = NULL;
+
+    GetModuleHandleExW(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS,
+	    (const char *) &tkIntStubs, &hInstance);
+    return hInstance;
+}
+#endif
+
