@@ -10,12 +10,10 @@
  * 	to display the cursor or not (e.g., readonly or disabled states);
  * 	TtkBlinkCursor() does not account for this.
  *
- * TODO:
- * 	Add script-level access to configure application-wide blink rate.
  */
 
 #include "tkInt.h"
-#include "ttkTheme.h"
+#include "ttkThemeInt.h"
 #include "ttkWidget.h"
 
 #define DEF_CURSOR_ON_TIME	600		/* milliseconds */
@@ -34,9 +32,12 @@ typedef struct
 /* CursorManagerDeleteProc --
  * 	InterpDeleteProc for cursor manager.
  */
-static void CursorManagerDeleteProc(ClientData clientData, Tcl_Interp *interp)
+static void CursorManagerDeleteProc(
+    void *clientData,
+    TCL_UNUSED(Tcl_Interp *))
 {
     CursorManager *cm = (CursorManager*)clientData;
+
     if (cm->timer) {
 	Tcl_DeleteTimerHandler(cm->timer);
     }
@@ -49,14 +50,38 @@ static void CursorManagerDeleteProc(ClientData clientData, Tcl_Interp *interp)
 static CursorManager *GetCursorManager(Tcl_Interp *interp)
 {
     static const char *cm_key = "ttk::CursorManager";
-    CursorManager *cm = Tcl_GetAssocData(interp, cm_key,0);
+    CursorManager *cm = (CursorManager *)Tcl_GetAssocData(interp, cm_key,0);
+    Tk_Window window;
+    Tk_Uid value;
+    int intValue;
 
     if (!cm) {
-	cm = ckalloc(sizeof(*cm));
+	cm = (CursorManager *)ckalloc(sizeof(*cm));
 	cm->timer = 0;
 	cm->owner = 0;
 	cm->onTime = DEF_CURSOR_ON_TIME;
 	cm->offTime = DEF_CURSOR_OFF_TIME;
+
+	/* Override on and off default times with values obtained from
+	 * the option database (if such values are specified).
+	 */
+
+	window = Tk_MainWindow(interp);
+	if (window) {
+	    value = Tk_GetOption(window, "insertOnTime", "OnTime");
+	    if (value) {
+		if (Tcl_GetInt(interp, value, &intValue) == TCL_OK) {
+		    cm->onTime = intValue;
+		}
+	    }
+	    value = Tk_GetOption(window, "insertOffTime", "OffTime");
+	    if (value) {
+		if (Tcl_GetInt(interp, value, &intValue) == TCL_OK) {
+		    cm->offTime = intValue;
+		}
+	    }
+	}
+
 	Tcl_SetAssocData(interp, cm_key, CursorManagerDeleteProc, cm);
     }
     return cm;
@@ -66,9 +91,9 @@ static CursorManager *GetCursorManager(Tcl_Interp *interp)
  *	Timer handler to blink the insert cursor on and off.
  */
 static void
-CursorBlinkProc(ClientData clientData)
+CursorBlinkProc(void *clientData)
 {
-    CursorManager *cm = (CursorManager*)clientData;
+    CursorManager *cm = (CursorManager *)clientData;
     int blinkTime;
 
     if (cm->owner->flags & CURSOR_ON) {
@@ -129,7 +154,7 @@ static void ClaimCursor(CursorManager *cm, WidgetCore *corePtr)
     (d == NotifyInferior || d == NotifyAncestor || d == NotifyNonlinear)
 
 static void
-CursorEventProc(ClientData clientData, XEvent *eventPtr)
+CursorEventProc(void *clientData, XEvent *eventPtr)
 {
     WidgetCore *corePtr = (WidgetCore *)clientData;
     CursorManager *cm = GetCursorManager(corePtr->interp);
@@ -152,6 +177,49 @@ CursorEventProc(ClientData clientData, XEvent *eventPtr)
     }
 }
 
+void TtkSetBlinkCursorOnTime(Tcl_Interp* interp, int onTime)
+{
+    CursorManager* cm = GetCursorManager(interp);
+
+    if (onTime >= 0)
+	cm->onTime = onTime;
+}
+
+void TtkSetBlinkCursorOffTime(Tcl_Interp* interp, int offTime)
+{
+    CursorManager* cm = GetCursorManager(interp);
+
+    if (offTime >= 0)
+	cm->offTime = offTime;
+}
+
+/*
+ * TtkSetBlinkCursorTimes --
+ * 	Set cursor blink on and off times from the "." style defaults
+ * 	-insertontime and -insertofftime - For instance to set cursor
+ * 	blinking off:
+ * 	    ttk::style configure . -insertofftime 0
+ */
+void TtkSetBlinkCursorTimes(Tcl_Interp* interp)
+{
+    Ttk_Theme theme;
+    Ttk_Style style = NULL;
+    Tcl_Obj* result;
+    int timeInterval;
+
+    theme = Ttk_GetCurrentTheme(interp);
+    style = Ttk_GetStyle(theme, ".");
+    result = Ttk_StyleDefault(style, "-insertontime");
+    if (result) {
+	Tcl_GetIntFromObj(interp, result, &timeInterval);
+	TtkSetBlinkCursorOnTime(interp, timeInterval);
+    }
+    result = Ttk_StyleDefault(style, "-insertofftime");
+    if (result) {
+	Tcl_GetIntFromObj(interp, result, &timeInterval);
+	TtkSetBlinkCursorOffTime(interp, timeInterval);
+    }
+}
 /*
  * TtkBlinkCursor (main routine) --
  * 	Arrange to blink the cursor on and off whenever the
