@@ -1,11 +1,10 @@
 /*
- * Copyright (c) 2003, Joe English
+ * Copyright © 2003 Joe English
  *
  * ttk::scrollbar widget.
  */
 
-#include <tk.h>
-
+#include "tkInt.h"
 #include "ttkTheme.h"
 #include "ttkWidget.h"
 
@@ -22,7 +21,7 @@ typedef struct
     double	first;			/* top fraction */
     double	last;			/* bottom fraction */
 
-    Ttk_Box	troughBox;		/* trough parcel */ 
+    Ttk_Box	troughBox;		/* trough parcel */
     int 	minSize;		/* minimum size of thumb */
 } ScrollbarPart;
 
@@ -32,15 +31,15 @@ typedef struct
     ScrollbarPart scrollbar;
 } Scrollbar;
 
-static Tk_OptionSpec ScrollbarOptionSpecs[] =
+static const Tk_OptionSpec ScrollbarOptionSpecs[] =
 {
     {TK_OPTION_STRING, "-command", "command", "Command", "",
-	Tk_Offset(Scrollbar,scrollbar.commandObj), -1, 0,0,0},
+	offsetof(Scrollbar, scrollbar.commandObj), TCL_INDEX_NONE, 0, 0, 0},
 
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient", "vertical",
-	Tk_Offset(Scrollbar,scrollbar.orientObj),
-	Tk_Offset(Scrollbar,scrollbar.orient),
-	0,(ClientData)ttkOrientStrings,STYLE_CHANGED },
+	offsetof(Scrollbar, scrollbar.orientObj),
+	offsetof(Scrollbar, scrollbar.orient),
+	0, ttkOrientStrings, STYLE_CHANGED },
 
     WIDGET_TAKEFOCUS_FALSE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
@@ -50,10 +49,13 @@ static Tk_OptionSpec ScrollbarOptionSpecs[] =
  * +++ Widget hooks.
  */
 
-static void 
-ScrollbarInitialize(Tcl_Interp *interp, void *recordPtr)
+static void
+ScrollbarInitialize(
+    TCL_UNUSED(Tcl_Interp *),
+    void *recordPtr)
 {
-    Scrollbar *sb = recordPtr;
+    Scrollbar *sb = (Scrollbar *)recordPtr;
+
     sb->scrollbar.first = 0.0;
     sb->scrollbar.last = 1.0;
 
@@ -63,7 +65,7 @@ ScrollbarInitialize(Tcl_Interp *interp, void *recordPtr)
 static Ttk_Layout ScrollbarGetLayout(
     Tcl_Interp *interp, Ttk_Theme theme, void *recordPtr)
 {
-    Scrollbar *sb = recordPtr;
+    Scrollbar *sb = (Scrollbar *)recordPtr;
     return TtkWidgetGetOrientedLayout(
 	interp, theme, recordPtr, sb->scrollbar.orientObj);
 }
@@ -77,7 +79,7 @@ static Ttk_Layout ScrollbarGetLayout(
  */
 static void ScrollbarDoLayout(void *recordPtr)
 {
-    Scrollbar *sb = recordPtr;
+    Scrollbar *sb = (Scrollbar *)recordPtr;
     WidgetCore *corePtr = &sb->core;
     Ttk_Element thumb;
     Ttk_Box thumbBox;
@@ -131,9 +133,9 @@ static void ScrollbarDoLayout(void *recordPtr)
  */
 static int
 ScrollbarSetCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    Scrollbar *scrollbar = recordPtr;
+    Scrollbar *scrollbar = (Scrollbar *)recordPtr;
     Tcl_Obj *firstObj, *lastObj;
     double first, last;
 
@@ -182,9 +184,9 @@ ScrollbarSetCommand(
  */
 static int
 ScrollbarGetCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    Scrollbar *scrollbar = recordPtr;
+    Scrollbar *scrollbar = (Scrollbar *)recordPtr;
     Tcl_Obj *result[2];
 
     if (objc != 2) {
@@ -205,9 +207,9 @@ ScrollbarGetCommand(
  */
 static int
 ScrollbarDeltaCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    Scrollbar *sb = recordPtr;
+    Scrollbar *sb = (Scrollbar *)recordPtr;
     double dx, dy;
     double delta = 0.0;
 
@@ -241,13 +243,13 @@ ScrollbarDeltaCommand(
 
 /* $sb fraction $x $y --
  * 	Returns a real number between 0 and 1 indicating  where  the
- * 	point given by x and y lies in the trough area of the scrollbar. 
+ * 	point given by x and y lies in the trough area of the scrollbar.
  */
 static int
 ScrollbarFractionCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    Scrollbar *sb = recordPtr;
+    Scrollbar *sb = (Scrollbar *)recordPtr;
     Ttk_Box b = sb->scrollbar.troughBox;
     int minSize = sb->scrollbar.minSize;
     double x, y;
@@ -280,8 +282,8 @@ ScrollbarFractionCommand(
 }
 
 static const Ttk_Ensemble ScrollbarCommands[] = {
-    { "configure",	TtkWidgetConfigureCommand,0 },
     { "cget",		TtkWidgetCgetCommand,0 },
+    { "configure",	TtkWidgetConfigureCommand,0 },
     { "delta",    	ScrollbarDeltaCommand,0 },
     { "fraction",    	ScrollbarFractionCommand,0 },
     { "get",    	ScrollbarGetCommand,0 },
@@ -289,13 +291,14 @@ static const Ttk_Ensemble ScrollbarCommands[] = {
     { "instate",	TtkWidgetInstateCommand,0 },
     { "set",  		ScrollbarSetCommand,0 },
     { "state",  	TtkWidgetStateCommand,0 },
+    { "style",		TtkWidgetStyleCommand,0 },
     { 0,0,0 }
 };
 
 /*------------------------------------------------------------------------
  * +++ Widget specification.
  */
-static WidgetSpec ScrollbarWidgetSpec =
+static const WidgetSpec ScrollbarWidgetSpec =
 {
     "TScrollbar",		/* className */
     sizeof(Scrollbar),		/* recordSize */
@@ -315,24 +318,22 @@ TTK_BEGIN_LAYOUT(VerticalScrollbarLayout)
     TTK_GROUP("Vertical.Scrollbar.trough", TTK_FILL_Y,
 	TTK_NODE("Vertical.Scrollbar.uparrow", TTK_PACK_TOP)
 	TTK_NODE("Vertical.Scrollbar.downarrow", TTK_PACK_BOTTOM)
-	TTK_NODE(
-	    "Vertical.Scrollbar.thumb", TTK_PACK_TOP|TTK_EXPAND|TTK_FILL_BOTH))
+	TTK_NODE("Vertical.Scrollbar.thumb", TTK_FILL_BOTH))
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(HorizontalScrollbarLayout)
     TTK_GROUP("Horizontal.Scrollbar.trough", TTK_FILL_X,
 	TTK_NODE("Horizontal.Scrollbar.leftarrow", TTK_PACK_LEFT)
 	TTK_NODE("Horizontal.Scrollbar.rightarrow", TTK_PACK_RIGHT)
-	TTK_NODE(
-	"Horizontal.Scrollbar.thumb", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_BOTH))
+	TTK_NODE("Horizontal.Scrollbar.thumb", TTK_FILL_BOTH))
 TTK_END_LAYOUT
 
 /*------------------------------------------------------------------------
  * +++ Initialization.
  */
 
-MODULE_SCOPE
-void TtkScrollbar_Init(Tcl_Interp *interp)
+MODULE_SCOPE void
+TtkScrollbar_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme = Ttk_GetDefaultTheme(interp);
 
