@@ -1,4 +1,4 @@
-/* square.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sourceforge.net>
+/* square.c - Copyright © 2004 Pat Thoyts <patthoyts@users.sourceforge.net>
  *
  * Minimal sample ttk widget.
  */
@@ -48,31 +48,31 @@ typedef struct
  * defined for all widgets.
  */
 
-static Tk_OptionSpec SquareOptionSpecs[] =
+static const Tk_OptionSpec SquareOptionSpecs[] =
 {
     {TK_OPTION_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
-     DEFAULT_BORDERWIDTH, Tk_Offset(Square,square.borderWidthObj), -1,
+     DEFAULT_BORDERWIDTH, offsetof(Square,square.borderWidthObj), TCL_INDEX_NONE,
      0,0,GEOMETRY_CHANGED },
     {TK_OPTION_BORDER, "-foreground", "foreground", "Foreground",
-     DEFAULT_BACKGROUND, Tk_Offset(Square,square.foregroundObj),
-     -1, 0, 0, 0},
+     DEFAULT_BACKGROUND, offsetof(Square,square.foregroundObj),
+     TCL_INDEX_NONE, 0, 0, 0},
 
     {TK_OPTION_PIXELS, "-width", "width", "Width",
-     "50", Tk_Offset(Square,square.widthObj), -1, 0, 0,
+     "50", offsetof(Square,square.widthObj), TCL_INDEX_NONE, 0, 0,
      GEOMETRY_CHANGED},
     {TK_OPTION_PIXELS, "-height", "height", "Height",
-     "50", Tk_Offset(Square,square.heightObj), -1, 0, 0,
+     "50", offsetof(Square,square.heightObj), TCL_INDEX_NONE, 0, 0,
      GEOMETRY_CHANGED},
 
     {TK_OPTION_STRING, "-padding", "padding", "Pad", NULL,
-     Tk_Offset(Square,square.paddingObj), -1,
+     offsetof(Square,square.paddingObj), TCL_INDEX_NONE,
      TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED },
 
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
-     NULL, Tk_Offset(Square,square.reliefObj), -1, TK_OPTION_NULL_OK, 0, 0},
+     NULL, offsetof(Square,square.reliefObj), TCL_INDEX_NONE, TK_OPTION_NULL_OK, 0, 0},
 
     {TK_OPTION_ANCHOR, "-anchor", "anchor", "Anchor",
-     "center", Tk_Offset(Square,square.anchorObj), -1, 0, 0, 0},
+     "center", offsetof(Square,square.anchorObj), TCL_INDEX_NONE, 0, 0, 0},
 
     WIDGET_TAKEFOCUS_TRUE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
@@ -129,11 +129,12 @@ SquareDoLayout(void *clientData)
  */
 
 static const Ttk_Ensemble SquareCommands[] = {
-    { "configure",	TtkWidgetConfigureCommand,0 },
     { "cget",		TtkWidgetCgetCommand,0 },
+    { "configure",	TtkWidgetConfigureCommand,0 },
     { "identify",	TtkWidgetIdentifyCommand,0 },
     { "instate",	TtkWidgetInstateCommand,0 },
     { "state",  	TtkWidgetStateCommand,0 },
+    { "style",		TtkWidgetStyleCommand,0 },
     { 0,0,0 }
 };
 
@@ -143,7 +144,7 @@ static const Ttk_Ensemble SquareCommands[] = {
  * with Tk in the package initialization code (see bottom).
  */
 
-static WidgetSpec SquareWidgetSpec =
+static const WidgetSpec SquareWidgetSpec =
 {
     "TSquare",			/* className */
     sizeof(Square),		/* recordSize */
@@ -176,18 +177,18 @@ typedef struct
     Tcl_Obj *heightObj;
 } SquareElement;
 
-static Ttk_ElementOptionSpec SquareElementOptions[] =
+static const Ttk_ElementOptionSpec SquareElementOptions[] =
 {
-    { "-background", TK_OPTION_BORDER, Tk_Offset(SquareElement,borderObj),
+    { "-background", TK_OPTION_BORDER, offsetof(SquareElement,borderObj),
     	DEFAULT_BACKGROUND },
-    { "-foreground", TK_OPTION_BORDER, Tk_Offset(SquareElement,foregroundObj),
+    { "-foreground", TK_OPTION_BORDER, offsetof(SquareElement,foregroundObj),
     	DEFAULT_BACKGROUND },
-    { "-borderwidth", TK_OPTION_PIXELS, Tk_Offset(SquareElement,borderWidthObj),
+    { "-borderwidth", TK_OPTION_PIXELS, offsetof(SquareElement,borderWidthObj),
     	DEFAULT_BORDERWIDTH },
-    { "-relief", TK_OPTION_RELIEF, Tk_Offset(SquareElement,reliefObj),
+    { "-relief", TK_OPTION_RELIEF, offsetof(SquareElement,reliefObj),
     	"raised" },
-    { "-width",  TK_OPTION_PIXELS, Tk_Offset(SquareElement,widthObj), "20"},
-    { "-height", TK_OPTION_PIXELS, Tk_Offset(SquareElement,heightObj), "20"},
+    { "-width",  TK_OPTION_PIXELS, offsetof(SquareElement,widthObj), "20"},
+    { "-height", TK_OPTION_PIXELS, offsetof(SquareElement,heightObj), "20"},
     { NULL, TK_OPTION_BOOLEAN, 0, NULL }
 };
 
@@ -238,7 +239,7 @@ static void SquareElementDraw(
 	b.x, b.y, b.width, b.height, borderWidth, relief);
 }
 
-static Ttk_ElementSpec SquareElementSpec =
+static const Ttk_ElementSpec SquareElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(SquareElement),
