@@ -1,11 +1,11 @@
 /*
- * Copyright (c) 2003, Joe English
+ * Copyright © 2003, Joe English
  *
  * Core widget utilities.
  */
 
 #include "tkInt.h"
-#include "ttkTheme.h"
+#include "ttkThemeInt.h"
 #include "ttkWidget.h"
 
 /*------------------------------------------------------------------------
@@ -81,15 +81,19 @@ static void EndDrawing(Tk_Window tkwin, Drawable d)
 #else
 /* No double-buffering: draw directly into the window. */
 static Drawable BeginDrawing(Tk_Window tkwin) { return Tk_WindowId(tkwin); }
-static void EndDrawing(Tk_Window tkwin, Drawable d) { }
+static void EndDrawing(
+    TCL_UNUSED(Tk_Window),
+    TCL_UNUSED(Drawable))
+{
+}
 #endif
 
 /* DrawWidget --
  *	Redraw a widget.  Called as an idle handler.
  */
-static void DrawWidget(ClientData recordPtr)
+static void DrawWidget(void *recordPtr)
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
 
     corePtr->flags &= ~REDISPLAY_PENDING;
     if (Tk_IsMapped(corePtr->tkwin)) {
@@ -121,9 +125,10 @@ void TtkRedisplayWidget(WidgetCore *corePtr)
  * 	Invoked whenever fonts or other system resources are changed;
  * 	recomputes geometry.
  */
-static void WidgetWorldChanged(ClientData clientData)
+static void WidgetWorldChanged(void *clientData)
 {
-    WidgetCore *corePtr = clientData;
+    WidgetCore *corePtr = (WidgetCore *)clientData;
+    (void)UpdateLayout(corePtr->interp, corePtr);
     SizeChanged(corePtr);
     TtkRedisplayWidget(corePtr);
 }
@@ -158,9 +163,9 @@ void TtkWidgetChangeState(WidgetCore *corePtr,
  */
 static int
 WidgetInstanceObjCmd(
-    ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = clientData;
+    WidgetCore *corePtr = (WidgetCore *)clientData;
     const Ttk_Ensemble *commands = corePtr->widgetSpec->commands;
     int status;
 
@@ -193,9 +198,9 @@ WidgetInstanceObjCmd(
  * 	Widget instance command	deletion callback.
  */
 static void
-WidgetInstanceObjCmdDeleted(ClientData clientData)
+WidgetInstanceObjCmdDeleted(void *clientData)
 {
-    WidgetCore *corePtr = clientData;
+    WidgetCore *corePtr = (WidgetCore *)clientData;
     corePtr->widgetCmd = NULL;
     if (corePtr->tkwin != NULL)
 	Tk_DestroyWindow(corePtr->tkwin);
@@ -212,7 +217,7 @@ DestroyWidget(WidgetCore *corePtr)
     corePtr->widgetSpec->cleanupProc(corePtr);
 
     Tk_FreeConfigOptions(
-	(ClientData)corePtr, corePtr->optionTable, corePtr->tkwin);
+	    corePtr, corePtr->optionTable, corePtr->tkwin);
 
     if (corePtr->layout) {
 	Ttk_FreeLayout(corePtr->layout);
@@ -257,7 +262,7 @@ static const unsigned CoreEventMask
     | LeaveWindowMask
     ;
 
-static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
+static void CoreEventProc(void *clientData, XEvent *eventPtr)
 {
     WidgetCore *corePtr = (WidgetCore *)clientData;
 
@@ -309,7 +314,6 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 	case VirtualEvent: {
 	    const char *name = ((XVirtualEvent *)eventPtr)->name;
 	    if ((name != NULL) && !strcmp("ThemeChanged", name)) {
-		(void)UpdateLayout(corePtr->interp, corePtr);
 		WidgetWorldChanged(corePtr);
 	    }
 	    break;
@@ -320,7 +324,7 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
     }
 }
 
-static Tk_ClassProcs widgetClassProcs = {
+static const Tk_ClassProcs widgetClassProcs = {
     sizeof(Tk_ClassProcs),	/* size */
     WidgetWorldChanged,		/* worldChangedProc */
     NULL,			/* createProc */
@@ -333,9 +337,9 @@ static Tk_ClassProcs widgetClassProcs = {
  *	ClientData is a WidgetSpec *.
  */
 int TtkWidgetConstructorObjCmd(
-    ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *clientData, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetSpec *widgetSpec = clientData;
+    WidgetSpec *widgetSpec = (WidgetSpec *)clientData;
     const char *className = widgetSpec->className;
     Tk_OptionTable optionTable =
 	Tk_CreateOptionTable(interp, widgetSpec->optionSpecs);
@@ -343,7 +347,7 @@ int TtkWidgetConstructorObjCmd(
     void *recordPtr;
     WidgetCore *corePtr;
     Tk_SavedOptions savedOptions;
-    int i;
+    Tcl_Size i;
 
     if (objc < 2 || objc % 2 == 1) {
 	Tcl_WrongNumArgs(interp, 1, objv, "pathName ?-option value ...?");
@@ -371,7 +375,7 @@ int TtkWidgetConstructorObjCmd(
      */
     recordPtr = ckalloc(widgetSpec->recordSize);
     memset(recordPtr, 0, widgetSpec->recordSize);
-    corePtr = recordPtr;
+    corePtr = (WidgetCore *)recordPtr;
 
     corePtr->tkwin	= tkwin;
     corePtr->interp 	= interp;
@@ -446,7 +450,7 @@ error:
 Ttk_Layout TtkWidgetGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     const char *styleName = 0;
 
     if (corePtr->styleObj)
@@ -468,10 +472,10 @@ Ttk_Layout TtkWidgetGetLayout(
 Ttk_Layout TtkWidgetGetOrientedLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr, Tcl_Obj *orientObj)
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     const char *baseStyleName = 0;
     Tcl_DString styleName;
-    int orient = TTK_ORIENT_HORIZONTAL;
+    Ttk_Orient orient = TTK_ORIENT_HORIZONTAL;
     Ttk_Layout layout;
 
     Tcl_DStringInit(&styleName);
@@ -480,9 +484,9 @@ Ttk_Layout TtkWidgetGetOrientedLayout(
      */
     Ttk_GetOrientFromObj(NULL, orientObj, &orient);
     if (orient == TTK_ORIENT_HORIZONTAL)
-	Tcl_DStringAppend(&styleName, "Horizontal.", -1);
+	Tcl_DStringAppend(&styleName, "Horizontal.", TCL_INDEX_NONE);
     else
-	Tcl_DStringAppend(&styleName, "Vertical.", -1);
+	Tcl_DStringAppend(&styleName, "Vertical.", TCL_INDEX_NONE);
 
     /* Add base style name:
      */
@@ -491,7 +495,7 @@ Ttk_Layout TtkWidgetGetOrientedLayout(
     if (!baseStyleName || *baseStyleName == '\0')
     	baseStyleName = corePtr->widgetSpec->className;
 
-    Tcl_DStringAppend(&styleName, baseStyleName, -1);
+    Tcl_DStringAppend(&styleName, baseStyleName, TCL_INDEX_NONE);
 
     /* Create layout:
      */
@@ -506,14 +510,19 @@ Ttk_Layout TtkWidgetGetOrientedLayout(
 /* TtkNullInitialize --
  * 	Default widget initializeProc (no-op)
  */
-void TtkNullInitialize(Tcl_Interp *interp, void *recordPtr)
+void TtkNullInitialize(
+    TCL_UNUSED(Tcl_Interp *),
+    TCL_UNUSED(void *))
 {
 }
 
 /* TtkNullPostConfigure --
  * 	Default widget postConfigureProc (no-op)
  */
-int TtkNullPostConfigure(Tcl_Interp *interp, void *clientData, int mask)
+int TtkNullPostConfigure(
+    TCL_UNUSED(Tcl_Interp *),
+    TCL_UNUSED(void *),
+    TCL_UNUSED(int))
 {
     return TCL_OK;
 }
@@ -524,7 +533,7 @@ int TtkNullPostConfigure(Tcl_Interp *interp, void *clientData, int mask)
  */
 int TtkCoreConfigure(Tcl_Interp *interp, void *clientData, int mask)
 {
-    WidgetCore *corePtr = clientData;
+    WidgetCore *corePtr = (WidgetCore *)clientData;
     int status = TCL_OK;
 
     if (mask & STYLE_CHANGED) {
@@ -537,7 +546,8 @@ int TtkCoreConfigure(Tcl_Interp *interp, void *clientData, int mask)
 /* TtkNullCleanup --
  * 	Default widget cleanupProc (no-op)
  */
-void TtkNullCleanup(void *recordPtr)
+void TtkNullCleanup(
+    TCL_UNUSED(void *))
 {
     return;
 }
@@ -547,7 +557,7 @@ void TtkNullCleanup(void *recordPtr)
  */
 void TtkWidgetDoLayout(void *clientData)
 {
-    WidgetCore *corePtr = clientData;
+    WidgetCore *corePtr = (WidgetCore *)clientData;
     Ttk_PlaceLayout(corePtr->layout,corePtr->state,Ttk_WinBox(corePtr->tkwin));
 }
 
@@ -556,7 +566,7 @@ void TtkWidgetDoLayout(void *clientData)
  */
 void TtkWidgetDisplay(void *recordPtr, Drawable d)
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Ttk_DrawLayout(corePtr->layout, corePtr->state, d);
 }
 
@@ -565,7 +575,7 @@ void TtkWidgetDisplay(void *recordPtr, Drawable d)
  */
 int TtkWidgetSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Ttk_LayoutSize(corePtr->layout, corePtr->state, widthPtr, heightPtr);
     return 1;
 }
@@ -577,9 +587,9 @@ int TtkWidgetSize(void *recordPtr, int *widthPtr, int *heightPtr)
 /* $w cget -option
  */
 int TtkWidgetCgetCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Tcl_Obj *result;
 
     if (objc != 3) {
@@ -597,9 +607,9 @@ int TtkWidgetCgetCommand(
 /* $w configure ?-option ?value ....??
  */
 int TtkWidgetConfigureCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Tcl_Obj *result;
 
     if (objc == 2) {
@@ -667,9 +677,9 @@ int TtkWidgetConfigureCommand(
  */
 
 int TtkWidgetStateCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Ttk_StateSpec spec;
     int status;
     Ttk_State oldState, changed;
@@ -707,9 +717,9 @@ int TtkWidgetStateCommand(
  */
 
 int TtkWidgetInstateCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Ttk_State state = corePtr->state;
     Ttk_StateSpec spec;
     int status = TCL_OK;
@@ -738,9 +748,9 @@ int TtkWidgetInstateCommand(
  * 	Returns: name of element at $x, $y
  */
 int TtkWidgetIdentifyCommand(
-    void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
 {
-    WidgetCore *corePtr = recordPtr;
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
     Ttk_Element element;
     static const char *const whatTable[] = { "element", NULL };
     int x, y, what;
@@ -772,4 +782,24 @@ int TtkWidgetIdentifyCommand(
     return TCL_OK;
 }
 
+/* $w style
+ * 	Return the style currently applied to the widget.
+ */
+
+int TtkWidgetStyleCommand(
+    void *recordPtr, Tcl_Interp *interp, Tcl_Size objc, Tcl_Obj *const objv[])
+{
+    WidgetCore *corePtr = (WidgetCore *)recordPtr;
+
+    if (objc != 2) {
+	Tcl_WrongNumArgs(interp, 2, objv, "");
+	return TCL_ERROR;
+    }
+
+    Tcl_SetObjResult(interp, Tcl_NewStringObj(
+	    Ttk_StyleName(Ttk_LayoutStyle(corePtr->layout)), -1));
+
+    return TCL_OK;
+}
+
 /*EOF*/
