@@ -3,7 +3,7 @@
 #	Some functions needed for the common dialog boxes. Probably need to go
 #	in a different file.
 #
-# Copyright (c) 1996 Sun Microsystems, Inc.
+# Copyright © 1996 Sun Microsystems, Inc.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -29,7 +29,8 @@
 #    {....}
 # }
 #
-# flags = currently unused.
+# flags = a list of flags. Currently supported flags are:
+#     DONTSETDEFAULTS = skip default values setting
 #
 # argList = The list of  "-option value" pairs.
 #
@@ -63,8 +64,10 @@ proc tclParseConfigSpec {w specs flags argList} {
 
     # 2: set the default values
     #
-    foreach cmdsw [array names cmd] {
-	set data($cmdsw) $def($cmdsw)
+    if {"DONTSETDEFAULTS" ni $flags} {
+        foreach cmdsw [array names cmd] {
+	    set data($cmdsw) $def($cmdsw)
+        }
     }
 
     # 3: parse the argument list
@@ -146,7 +149,7 @@ proc ::tk::FocusGroup_BindIn {t w cmd} {
     variable ::tk::Priv
     if {![info exists Priv(fg,$t)]} {
 	return -code error -errorcode [list TK LOOKUP FOCUS_GROUP $t] \
-	    "focus group \"$t\" doesn't exist"
+	    "focus group \"$t\" does not exist"
     }
     set FocusIn($t,$w) $cmd
 }
@@ -163,7 +166,7 @@ proc ::tk::FocusGroup_BindOut {t w cmd} {
     variable ::tk::Priv
     if {![info exists Priv(fg,$t)]} {
 	return -code error -errorcode [list TK LOOKUP FOCUS_GROUP $t] \
-	    "focus group \"$t\" doesn't exist"
+	    "focus group \"$t\" does not exist"
     }
     set FocusOut($t,$w) $cmd
 }
