@@ -7,7 +7,15 @@ if {![info exists widgetDemo]} {
     error "This script should be run from the \"widget\" demo."
 }
 
-package require Tk
+package require tk
+
+# Make an Aqua button's fill color match its parent's background
+proc blend {bt} {
+    if {[tk windowingsystem] eq "aqua"} {
+	$bt configure -highlightbackground [[winfo parent $bt] cget -background]
+    }
+    return $bt
+}
 
 set w .twind
 catch {destroy $w}
@@ -29,7 +37,7 @@ ttk::scrollbar $w.scroll -command "$t yview"
 pack $w.scroll -side right -fill y
 panedwindow $w.pane
 pack $w.pane -expand yes -fill both
-$w.pane add $w.f
+$w.pane add $w.f -stretch always
 # Import to raise given creation order above
 raise $w.f
 
@@ -53,17 +61,19 @@ $t insert end "it.  These are called \"embedded windows\", "
 $t insert end "and they can consist of arbitrary widgets.  "
 $t insert end "For example, here are two embedded button "
 $t insert end "widgets.  You can click on the first button to "
-$t window create end -window $t.on
+$t window create end -window [blend $t.on]
 $t insert end " horizontal scrolling, which also turns off "
 $t insert end "word wrapping.  Or, you can click on the second "
 $t insert end "button to\n"
-$t window create end -window $t.off
+$t window create end -window [blend $t.off]
 $t insert end " horizontal scrolling and turn back on word wrapping.\n\n"
 
 $t insert end "Or, here is another example.  If you "
 $t window create end -create {
     button %W.click -text "Click Here" -command "textWindPlot %W" \
-	    -cursor top_left_arrow}
+	-cursor top_left_arrow
+    blend %W.click
+}
 
 $t insert end " a canvas displaying an x-y plot will appear right here."
 $t mark set plot insert
@@ -72,7 +82,8 @@ $t insert end "  You can drag the data points around with the mouse, "
 $t insert end "or you can click here to "
 $t window create end -create {
     button %W.delete -text "Delete" -command "textWindDel %W" \
-	    -cursor top_left_arrow
+	-cursor top_left_arrow
+    blend %W.delete
 }
 $t insert end " the plot again.\n\n"
 
@@ -80,19 +91,21 @@ $t insert end "You can also create multiple text widgets each of which "
 $t insert end "display the same underlying text. Click this button to "
 $t window create end \
   -create {button %W.peer -text "Make A Peer" -command "textMakePeer %W" \
-  -cursor top_left_arrow} -padx 3
+	       -cursor top_left_arrow
+      blend %W.peer} -padx 3p
 $t insert end " widget.  Notice how peer widgets can have different "
 $t insert end "font settings, and by default contain all the images "
-$t insert end "of the 'parent', but many of the embedded windows, "
-$t insert end "such as buttons will not be there.  The easiest way "
-$t insert end "to ensure they are in all peers is to use '-create' "
-$t insert end "embedded window creation scripts "
-$t insert end "(the plot above and the 'Make A Peer' button are "
-$t insert end "designed to show up in all peers).  A good use of "
+$t insert end "of the 'parent', but that the embedded windows, "
+$t insert end "such as buttons may not appear in the peer.  To ensure "
+$t insert end "that embedded windows appear in all peers you can set the "
+$t insert end "'-create' option to a script or a string containing %W.  "
+$t insert end "(The plot above and the 'Make A Peer' button are "
+$t insert end "designed to show up in all peers.)  A good use of "
 $t insert end "peers is for "
 $t window create end \
   -create {button %W.split -text "Split Windows" -command "textSplitWindow %W" \
-  -cursor top_left_arrow} -padx 3
+	       -cursor top_left_arrow
+      blend %W.split} -padx 3p
 $t insert end " \n\n"
 
 $t insert end "Users of previous versions of Tk will also be interested "
@@ -112,26 +125,27 @@ $t insert end "you can see how the text widget automatically "
 $t insert end "changes the layout.  Click on the button again "
 $t insert end "to restore the short string.\n"
 
+$t insert end "\nNOTE: these buttons will not appear in peers!\n" "peer_warning"
 button $t.default -text Default -command "embDefBg $t" \
 	-cursor top_left_arrow
-$t window create end -window $t.default -padx 3
+$t window create end -window $t.default -padx 3p
 global embToggle
 set embToggle Short
 checkbutton $t.toggle -textvariable embToggle -indicatoron 0 \
 	-variable embToggle -onvalue "A much longer string" \
-	-offvalue "Short" -cursor top_left_arrow -pady 5 -padx 2
-$t window create end -window $t.toggle -padx 3 -pady 2
+	-offvalue "Short" -cursor top_left_arrow -pady 3p -padx 1.5p
+$t window create end -window $t.toggle -padx 3p -pady 1.5p
 set i 1
 foreach color {AntiqueWhite3 Bisque1 Bisque2 Bisque3 Bisque4
 	SlateBlue3 RoyalBlue1 SteelBlue2 DeepSkyBlue3 LightBlue1
 	DarkSlateGray1 Aquamarine2 DarkSeaGreen2 SeaGreen1
 	Yellow1 IndianRed1 IndianRed2 Tan1 Tan4} {
     button $t.color$i -text $color -cursor top_left_arrow -command \
-	    "$t configure -bg $color"
-    $t window create end -window $t.color$i -padx 3 -pady 2
+	    "changeBg $t $color"
+    $t window create end -window [blend $t.color$i] -padx 3p -pady 1.5p
     incr i
 }
-$t tag add buttons $t.default end
+$t tag add buttons [blend $t.default] end
 
 button $t.bigB -text "Big borders" -command "textWindBigB $t" \
   -cursor top_left_arrow
@@ -152,29 +166,35 @@ set text_normal(pad) [$t cget -padx]
 
 $t insert end "\nYou can also change the usual border width and "
 $t insert end "highlightthickness and padding.\n"
-$t window create end -window $t.bigB
-$t window create end -window $t.smallB
-$t window create end -window $t.bigH
-$t window create end -window $t.smallH
-$t window create end -window $t.bigP
-$t window create end -window $t.smallP
+$t window create end -window [blend $t.bigB]
+$t window create end -window [blend $t.smallB]
+$t window create end -window [blend $t.bigH]
+$t window create end -window [blend $t.smallH]
+$t window create end -window [blend $t.bigP]
+$t window create end -window [blend $t.smallP]
 
 $t insert end "\n\nFinally, images fit comfortably in text widgets too:"
 
-$t image create end -image \
-    [image create photo -file [file join $tk_demoDirectory images ouster.png]]
+image create photo img -file [file join $tk_demoDirectory images ouster.png]
 
+# Create a copy of the image just created, magnified according to the
+# display's DPI scaling level.  Since the zooom factor must be an integer,
+# the copy will only be effectively magnified if $tk::scalingPct >= 200.
+image create photo img2
+img2 copy img -zoom [expr {$tk::scalingPct / 100}]
+
+$t image create end -image img2
 
 proc textWindBigB w {
-    $w configure -borderwidth 15
+    $w configure -borderwidth 12p
 }
 
 proc textWindBigH w {
-    $w configure -highlightthickness 15
+    $w configure -highlightthickness 12p
 }
 
 proc textWindBigP w {
-    $w configure -padx 15 -pady 15
+    $w configure -padx 12p -pady 12p
 }
 
 proc textWindSmallB w {
@@ -189,7 +209,6 @@ proc textWindSmallP w {
     $w configure -padx $::text_normal(pad) -pady $::text_normal(pad)
 }
 
-
 proc textWindOn w {
     catch {destroy $w.scroll2}
     set t $w.f.text
@@ -253,9 +272,9 @@ proc createPlot {t} {
 	$c addtag point withtag $item
     }
 
-    $c bind point <Any-Enter> "$c itemconfig current -fill red"
-    $c bind point <Any-Leave> "$c itemconfig current -fill SkyBlue2"
-    $c bind point <1> "embPlotDown $c %x %y"
+    $c bind point <Enter> "$c itemconfig current -fill red"
+    $c bind point <Leave> "$c itemconfig current -fill SkyBlue2"
+    $c bind point <Button-1> "embPlotDown $c %x %y"
     $c bind point <ButtonRelease-1> "$c dtag selected"
     bind $c <B1-Motion> "embPlotMove $c %x %y"
     return $c
@@ -290,8 +309,20 @@ proc textWindDel t {
     }
 }
 
+proc changeBg {t c} {
+    $t configure -background $c
+    if {[tk windowingsystem] eq "aqua"} {
+	foreach b [$t window names] {
+	    if {[winfo class $b] eq "Button"} {
+		$b configure -highlightbackground $c
+	    }
+	}
+    }
+}
+
 proc embDefBg t {
-    $t configure -background [lindex [$t configure -background] 3]
+    set bg [lindex [$t configure -background] 3]
+    changeBg $t $bg
 }
 
 proc textMakePeer {parent} {
@@ -302,6 +333,7 @@ proc textMakePeer {parent} {
     frame $w.f -highlightthickness 1 -borderwidth 1 -relief sunken
     set t [$parent peer create $w.f.text -yscrollcommand "$w.scroll set" \
 	       -borderwidth 0 -highlightthickness 0]
+    $t tag configure peer_warning -font boldFont
     pack $t -expand  yes -fill both
     ttk::scrollbar $w.scroll -command "$t yview"
     pack $w.scroll -side right -fill y
@@ -317,7 +349,8 @@ proc textSplitWindow {textW} {
 	    set w [winfo parent $parent]
 	    set t [$textW peer create $w.peer \
 	      -yscrollcommand "$w.scroll set"]
-	    $w.pane add $t
+	    $t tag configure peer_warning -font boldFont
+	    $w.pane add $t -stretch always
 	}
     } else {
         return
