@@ -3,7 +3,7 @@
 #	Implements messageboxes for platforms that do not have native
 #	messagebox support.
 #
-# Copyright (c) 1994-1997 Sun Microsystems, Inc.
+# Copyright © 1994-1997 Sun Microsystems, Inc.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -111,7 +111,7 @@ static unsigned char w3_bits[] = {
    0x00, 0xc0, 0x03, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};"
-
+
 # ::tk::MessageBox --
 #
 #	Pops up a messagebox with an application-supplied message with
@@ -123,13 +123,16 @@ static unsigned char w3_bits[] = {
 #	Color icons are used on Unix displays that have a color
 #	depth of 4 or more and $tk_strictMotif is not on.
 #
+#	Uses ::tk::Priv.${disp}(button) instead of ::tk::Priv(button) to
+#	avoid adverse effects of [::tk::ScreenChanged]. Bug [e2cec2fa41].
+#
 #	This procedure is a private procedure shouldn't be called
 #	directly. Call tk_messageBox instead.
 #
 #	See the user documentation for details on what tk_messageBox does.
 #
 proc ::tk::MessageBox {args} {
-    global tcl_platform tk_strictMotif
+    global tk_strictMotif
     variable ::tk::Priv
 
     set w ::tk::PrivMsgBox
@@ -137,12 +140,13 @@ proc ::tk::MessageBox {args} {
 
     #
     # The default value of the title is space (" ") not the empty string
-    # because for some window managers, a 
+    # because for some window managers, a
     #		wm title .foo ""
     # causes the window title to be "foo" instead of the empty string.
     #
     set specs {
 	{-default "" "" ""}
+	{-detail "" "" ""}
         {-icon "" "" "info"}
         {-message "" "" ""}
         {-parent "" "" .}
@@ -152,8 +156,9 @@ proc ::tk::MessageBox {args} {
 
     tclParseConfigSpec $w $specs "" $args
 
-    if {[lsearch -exact {info warning error question} $data(-icon)] == -1} {
-	error "bad -icon value \"$data(-icon)\": must be error, info, question, or warning"
+    if {$data(-icon) ni {info warning error question}} {
+	return -code error -errorcode [list TK LOOKUP ICON $data(-icon)] \
+	    "bad -icon value \"$data(-icon)\": must be error, info, question, or warning"
     }
     set windowingsystem [tk windowingsystem]
     if {$windowingsystem eq "aqua"} {
@@ -162,17 +167,39 @@ proc ::tk::MessageBox {args} {
 	    "warning"   {set data(-icon) "caution"}
 	    "info"      {set data(-icon) "note"}
 	}
-	option add *Dialog*background systemDialogBackgroundActive widgetDefault
-	option add *Dialog*Button.highlightBackground \
-		systemDialogBackgroundActive widgetDefault
     }
 
     if {![winfo exists $data(-parent)]} {
-	error "bad window path name \"$data(-parent)\""
+	return -code error -errorcode [list TK LOOKUP WINDOW $data(-parent)] \
+	    "bad window path name \"$data(-parent)\""
     }
 
+    # Select the vwait variable carefully.
+    set oldScreen $Priv(screen)
+    set screen [winfo screen $data(-parent)]
+
+    # Extract the display name (cf. ScreenChanged, including [Bug 2912473] fix).
+    set disp [string range $screen 0 [string last . $screen]-1]
+
+    # Ensure that namespace separators never occur in the display name (as
+    # they cause problems in variable names). Double-colons exist in some VNC
+    # display names. [Bug 2912473]
+    set disp [string map {:: _doublecolon_} $disp]
+
+    if {![info exists ::tk::Priv.${disp}]} {
+	# Use ScreenChanged to create ::tk::Priv.${disp}, then change back to old
+	# screen to avoid interfering with Tk expectations for bindings.
+	ScreenChanged $screen
+	ScreenChanged $oldScreen
+    }
+
+    variable ::tk::Priv.${disp}
+    # Now in place of ::tk::Priv(button), use ::tk::Priv.${disp}(button) which
+    # is the intended target variable of upvar and will not be redefined when
+    # ::tk::ScreenChanged is called.
+
     switch -- $data(-type) {
-	abortretryignore { 
+	abortretryignore {
 	    set names [list abort retry ignore]
 	    set labels [list &Abort &Retry &Ignore]
 	    set cancel abort
@@ -203,9 +230,10 @@ proc ::tk::MessageBox {args} {
 	    set cancel cancel
 	}
 	default {
-	    error "bad -type value \"$data(-type)\": must be\
-		    abortretryignore, ok, okcancel, retrycancel,\
-		    yesno, or yesnocancel"
+	    return -code error -errorcode [list TK LOOKUP DLG_TYPE $data(-type)] \
+		"bad -type value \"$data(-type)\": must be\
+		abortretryignore, ok, okcancel, retrycancel,\
+		yesno, or yesnocancel"
 	}
     }
 
@@ -214,7 +242,7 @@ proc ::tk::MessageBox {args} {
 	lappend buttons [list $name -text [mc $lab]]
     }
 
-    # If no default button was specified, the default default is the 
+    # If no default button was specified, the default default is the
     # first button (Bug: 2218).
 
     if {$data(-default) eq ""} {
@@ -229,7 +257,9 @@ proc ::tk::MessageBox {args} {
 	}
     }
     if {!$valid} {
-	error "invalid default button \"$data(-default)\""
+	return -code error -errorcode {TK MSGBOX DEFAULT} \
+	    "bad -default value \"$data(-default)\": must be\
+	    abort, retry, ignore, ok, cancel, no, or yes"
     }
 
     # 2. Set the dialog to be a child window of $parent
@@ -241,16 +271,17 @@ proc ::tk::MessageBox {args} {
 	set w .__tk__messagebox
     }
 
+    # There is only one background colour for the whole dialog
+    set bg [ttk::style lookup . -background]
+
     # 3. Create the top-level window and divide it into top
     # and bottom parts.
 
     catch {destroy $w}
-    toplevel $w -class Dialog
+    toplevel $w -class Dialog -bg $bg
     wm title $w $data(-title)
     wm iconname $w Dialog
     wm protocol $w WM_DELETE_WINDOW [list $w.$cancel invoke]
-    # There is only one background colour for the whole dialog
-    set bg [$w cget -background]
 
     # Message boxes should be transient with respect to their parent so that
     # they always stay on top of the parent window.  But some window managers
@@ -269,71 +300,55 @@ proc ::tk::MessageBox {args} {
         wm attributes $w -type dialog
     }
 
-    frame $w.bot -background $bg
+    ttk::frame $w.bot
+    grid anchor $w.bot center
     pack $w.bot -side bottom -fill both
-    frame $w.top -background $bg
+    ttk::frame $w.top
     pack $w.top -side top -fill both -expand 1
-    if {$windowingsystem ne "aqua"} {
-	$w.bot configure -relief raised -bd 1
-	$w.top configure -relief raised -bd 1
-    }
 
-    # 4. Fill the top part with bitmap and message (use the option
-    # database for -wraplength and -font so that they can be
+    # 4. Fill the top part with bitmap, message and detail (use the
+    # option database for -wraplength and -font so that they can be
     # overridden by the caller).
 
     option add *Dialog.msg.wrapLength 3i widgetDefault
-    if {$windowingsystem eq "aqua"} {
-	option add *Dialog.msg.font system widgetDefault
-    } else {
-	option add *Dialog.msg.font {Times 14} widgetDefault
-    }
+    option add *Dialog.dtl.wrapLength 3i widgetDefault
+    option add *Dialog.msg.font TkCaptionFont widgetDefault
+    option add *Dialog.dtl.font TkDefaultFont widgetDefault
 
-    label $w.msg -anchor nw -justify left -text $data(-message) \
-	    -background $bg
+    ttk::label $w.msg -anchor nw -justify left -text $data(-message)
+    if {$data(-detail) ne ""} {
+	ttk::label $w.dtl -anchor nw -justify left -text $data(-detail)
+    }
     if {$data(-icon) ne ""} {
-	if {($windowingsystem eq "aqua")
-		|| ([winfo depth $w] < 4) || $tk_strictMotif} {
+	if {([winfo depth $w] < 4) || $tk_strictMotif} {
+	    # ttk::label has no -bitmap option
 	    label $w.bitmap -bitmap $data(-icon) -background $bg
 	} else {
-	    canvas $w.bitmap -width 32 -height 32 -highlightthickness 0 \
-		    -background $bg
 	    switch $data(-icon) {
-		error {
-		    $w.bitmap create oval 0 0 31 31 -fill red -outline black
-		    $w.bitmap create line 9 9 23 23 -fill white -width 4
-		    $w.bitmap create line 9 23 23 9 -fill white -width 4
-		}
-		info {
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::b1
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::b2
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::i
-		}
-		question {
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::b1
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::b2
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::q
-		}
-		default {
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::w1
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::w2
-		    $w.bitmap create image 0 0 -anchor nw \
-			    -image ::tk::dialog::w3
-		}
+                error {
+                    ttk::label $w.bitmap -image ::tk::icons::error
+                }
+                info {
+                    ttk::label $w.bitmap -image ::tk::icons::information
+                }
+                question {
+                    ttk::label $w.bitmap -image ::tk::icons::question
+                }
+                default {
+                    ttk::label $w.bitmap -image ::tk::icons::warning
+                }
 	    }
 	}
     }
     grid $w.bitmap $w.msg -in $w.top -sticky news -padx 2m -pady 2m
+    grid configure $w.bitmap -sticky nw
     grid columnconfigure $w.top 1 -weight 1
-    grid rowconfigure $w.top 0 -weight 1
+    if {$data(-detail) ne ""} {
+	grid ^ $w.dtl -in $w.top -sticky news -padx 2m -pady {0 2m}
+	grid rowconfigure $w.top 1 -weight 1
+    } else {
+	grid rowconfigure $w.top 0 -weight 1
+    }
 
     # 5. Create a row of buttons at the bottom of the dialog.
 
@@ -347,8 +362,8 @@ proc ::tk::MessageBox {args} {
 	    set opts [list -text $capName]
 	}
 
-	eval [list tk::AmpWidget button $w.$name -padx 3m] $opts \
-		[list -command [list set tk::Priv(button) $name]]
+	eval [list tk::AmpWidget ttk::button $w.$name] $opts \
+		[list -command [list set tk::Priv.${disp}(button) $name]]
 
 	if {$name eq $data(-default)} {
 	    $w.$name configure -default active
@@ -382,28 +397,31 @@ proc ::tk::MessageBox {args} {
 
     if {$data(-default) ne ""} {
 	bind $w <FocusIn> {
-	    if {"Button" eq [winfo class %W]} {
+	    if {[winfo class %W] in "Button TButton"} {
 		%W configure -default active
 	    }
 	}
 	bind $w <FocusOut> {
-	    if {"Button" eq [winfo class %W]} {
+	    if {[winfo class %W] in "Button TButton"} {
 		%W configure -default normal
 	    }
 	}
     }
 
-    # 6. Create bindings for <Return> and <Escape> on the dialog
+    # 6. Create bindings for <Return>, <Escape> and <Destroy> on the dialog
 
     bind $w <Return> {
-	if {"Button" eq [winfo class %W]} {
-	    tk::ButtonInvoke %W
+	if {[winfo class %W] in "Button TButton"} {
+	    %W invoke
 	}
     }
 
     # Invoke the designated cancelling operation
     bind $w <Escape> [list $w.$cancel invoke]
 
+    # At <Destroy> the buttons have vanished, so must do this directly.
+    bind $w.msg <Destroy> [list set tk::Priv.${disp}(button) $cancel]
+
     # 7. Withdraw the window, then update all the geometry information
     # so we know how big it wants to be, then center the window in the
     # display (Motif style) and de-iconify it.
@@ -425,9 +443,12 @@ proc ::tk::MessageBox {args} {
     # may take the focus away so we can't redirect it.  Finally,
     # restore any grab that was in effect.
 
-    vwait ::tk::Priv(button)
+    vwait ::tk::Priv.${disp}(button)
+    # Copy the result now so any <Destroy> that happens won't cause
+    # trouble
+    set result [set Priv.${disp}(button)]
 
     ::tk::RestoreFocusGrab $w $focus
 
-    return $Priv(button)
+    return $result
 }
