@@ -3,7 +3,7 @@
 # This file contains procedures that change the color palette used
 # by Tk.
 #
-# Copyright (c) 1995-1997 Sun Microsystems, Inc.
+# Copyright © 1995-1997 Sun Microsystems, Inc.
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -71,6 +71,20 @@ proc ::tk_setPalette {args} {
     if {![info exists new(highlightBackground)]} {
 	set new(highlightBackground) $new(background)
     }
+    # 'buttonBackground' is the background color of the buttons in
+    # the spinbox widget.
+    if {![info exists new(buttonBackground)]} {
+	set new(buttonBackground) $new(background)
+    }
+    # 'selectColor' is the background of check & radio buttons.
+    if {![info exists new(selectColor)]} {
+	foreach {r g b} $bg {break}
+	if {$r+1.5*$g+0.5*$b > 100000} {
+	    set new(selectColor) white
+	} else {
+	    set new(selectColor) black
+	}
+    }
     if {![info exists new(activeBackground)]} {
 	# Pick a default active background that islighter than the
 	# normal background.  To do this, round each color component
@@ -100,7 +114,7 @@ proc ::tk_setPalette {args} {
 	set new(troughColor) $darkerBg
     }
 
-    # let's make one of each of the widgets so we know what the 
+    # let's make one of each of the widgets so we know what the
     # defaults are currently for this platform.
     toplevel .___tk_set_palette
     wm withdraw .___tk_set_palette
@@ -113,12 +127,12 @@ proc ::tk_setPalette {args} {
     }
 
     # Walk the widget hierarchy, recoloring all existing windows.
-    # The option database must be set according to what we do here, 
-    # but it breaks things if we set things in the database while 
+    # The option database must be set according to what we do here,
+    # but it breaks things if we set things in the database while
     # we are changing colors...so, ::tk::RecolorTree now returns the
     # option database changes that need to be made, and they
     # need to be evalled here to take effect.
-    # We have to walk the whole widget tree instead of just 
+    # We have to walk the whole widget tree instead of just
     # relying on the widgets we've created above to do the work
     # because different extensions may provide other kinds
     # of widgets that we don't currently know about, so we'll
@@ -139,12 +153,32 @@ proc ::tk_setPalette {args} {
     # next time we change the options.
 
     array set ::tk::Palette [array get new]
+
+    if {[tk windowingsystem] ne "x11" || [ttk::style theme use] ne "default"} {
+	return
+    }
+
+    # Update the 'default' ttk theme with the new palette,
+    # and then set 'default' as the current ttk theme,
+    # in order to apply the new palette to the ttk widgets.
+
+    foreach option [array names new] {
+	if {[info exists ttk::theme::default::colorOptionLookup($option)]} {
+	    foreach colorName $ttk::theme::default::colorOptionLookup($option) {
+		set ttk::theme::default::colors($colorName) $new($option)
+	    }
+	}
+    }
+    ttk::theme::default::reconfigureDefaultTheme
+    ttk::setTheme default
+
+    return
 }
 
 # ::tk::RecolorTree --
 # This procedure changes the colors in a window and all of its
 # descendants, according to information provided by the colors
-# argument. This looks at the defaults provided by the option 
+# argument. This looks at the defaults provided by the option
 # database, if it exists, and if not, then it looks at the default
 # value of the widget itself.
 #
@@ -155,6 +189,9 @@ proc ::tk_setPalette {args} {
 #			which contains color information.  Each element
 #			is named after a widget configuration option, and
 #			each value is the value for that option.
+# Return Value:
+#                       A list of commands which can be run to update
+#                       the defaults database when exec'ed.
 
 proc ::tk::RecolorTree {w colors} {
     upvar $colors c
@@ -166,11 +203,14 @@ proc ::tk::RecolorTree {w colors} {
     foreach dbOption [array names c] {
 	set option -[string tolower $dbOption]
 	set class [string replace $dbOption 0 0 [string toupper \
-		[string index $dbOption 0]]]
+	     [string index $dbOption 0]]]
+	# Make sure this option is valid for this window.
 	if {![catch {$w configure $option} value]} {
-	    # if the option database has a preference for this
-	    # dbOption, then use it, otherwise use the defaults
-	    # for the widget.
+	    # Update the option for this window.
+	    $w configure $option $c($dbOption)
+	    # Retrieve a default value for this option.  First check
+	    # the option database. If it is not in the database use
+	    # the value for the temporary prototype widget.
 	    set defaultcolor [option get $w $dbOption $class]
 	    if {$defaultcolor eq "" || \
 		    ([info exists prototype] && \
@@ -180,16 +220,15 @@ proc ::tk::RecolorTree {w colors} {
 	    if {$defaultcolor ne ""} {
 		set defaultcolor [winfo rgb . $defaultcolor]
 	    }
-	    set chosencolor [lindex $value 4]
-	    if {$chosencolor ne ""} {
-		set chosencolor [winfo rgb . $chosencolor]
+	    # If the color requested for this option differs from
+	    # the default, append a command to update the default.
+	    set requestcolor [lindex $value 4]
+	    if {$requestcolor ne ""} {
+		set requestcolor [winfo rgb . $requestcolor]
 	    }
-	    if {[string match $defaultcolor $chosencolor]} {
-		# Change the option database so that future windows will get
-		# the same colors.
+	    if {![string match $defaultcolor $requestcolor]} {
 		append result ";\noption add [list \
 		    *[winfo class $w].$dbOption $c($dbOption) 60]"
-		$w configure $option $c($dbOption)
 	    }
 	}
     }
@@ -205,27 +244,27 @@ proc ::tk::RecolorTree {w colors} {
 #
 # Arguments:
 # color -	Name of starting color.
-# perecent -	Integer telling how much to brighten or darken as a
+# percent -	Integer telling how much to brighten or darken as a
 #		percent: 50 means darken by 50%, 110 means brighten
 #		by 10%.
 
 proc ::tk::Darken {color percent} {
-    foreach {red green blue} [winfo rgb . $color] {
-	set red [expr {($red/256)*$percent/100}]
-	set green [expr {($green/256)*$percent/100}]
-	set blue [expr {($blue/256)*$percent/100}]
-	break
+    if {$percent < 0} {
+        return #000000
+    } elseif {$percent > 200} {
+        return #ffffff
+    } elseif {$percent <= 100} {
+        lassign [winfo rgb . $color] r g b
+        set r [expr {($r/256)*$percent/100}]
+        set g [expr {($g/256)*$percent/100}]
+        set b [expr {($b/256)*$percent/100}]
+    } elseif {$percent > 100} {
+        lassign [winfo rgb . $color] r g b
+        set r [expr {255 - ((65535-$r)/256)*(200-$percent)/100}]
+        set g [expr {255 - ((65535-$g)/256)*(200-$percent)/100}]
+        set b [expr {255 - ((65535-$b)/256)*(200-$percent)/100}]
     }
-    if {$red > 255} {
-	set red 255
-    }
-    if {$green > 255} {
-	set green 255
-    }
-    if {$blue > 255} {
-	set blue 255
-    }
-    return [format "#%02x%02x%02x" $red $green $blue]
+    return [format #%02x%02x%02x $r $g $b]
 }
 
 # ::tk_bisque --
