@@ -11,11 +11,10 @@ namespace eval ttk::theme::aqua {
 	    -foreground systemLabelColor \
 	    -selectbackground systemSelectedTextBackgroundColor \
 	    -selectforeground systemSelectedTextColor \
-	    -selectborderwidth 0 \
 	    -insertwidth 1
 
 	ttk::style map . \
-	    -foreground { 
+	    -foreground {
 		disabled systemDisabledControlTextColor
 		background systemLabelColor} \
 	    -selectbackground {
@@ -26,15 +25,75 @@ namespace eval ttk::theme::aqua {
 		!focus systemSelectedTextColor}
 
 	# Button
-	ttk::style configure TButton -anchor center -width -6 \
+	ttk::style configure TButton -anchor center \
 	    -foreground systemControlTextColor
 	ttk::style map TButton \
 	    -foreground {
 		pressed white
-	        {alternate !pressed !background} white}
+	        {alternate !pressed !background} white
+	        disabled systemDisabledControlTextColor}
+
+	# Menubutton
 	ttk::style configure TMenubutton -anchor center -padding {2 0 0 2}
+
+	# Toolbutton
 	ttk::style configure Toolbutton -anchor center
 
+	# Inline Button
+	ttk::style configure InlineButton -anchor center -font TkHeadingFont \
+	    -foreground systemTextBackgroundColor
+	ttk::style map InlineButton \
+	    -foreground {
+		disabled systemWindowBackgroundColor
+	    }
+
+	# Image Button
+	ttk::style configure ImageButton -anchor center -width 1 \
+	    -compound top
+	ttk::style map ImageButton \
+	    -foreground {
+		pressed systemLabelColor
+		!pressed systemSecondaryLabelColor
+	    }
+
+	# Recessed (radio) button
+	font create RecessedFont -family EmphasizedSystem -size 11 -weight bold
+	ttk::style configure RecessedButton \
+	    -foreground systemControlTextColor
+	ttk::style map RecessedButton \
+	    -foreground {
+		{disabled selected} systemWindowBackgroundColor3
+		{disabled !selected} systemDisabledControlTextColor
+		selected systemTextBackgroundColor
+		active white
+		pressed white
+	    } \
+	    -font {
+		selected RecessedFont
+		active RecessedFont
+		pressed RecessedFont
+	    }
+
+	# Sidebar (radio) button
+	font create SidebarFont -family .AppleSystemUIFont -size 11 -weight normal
+	ttk::style configure SidebarButton \
+	    -foreground systemControlTextColor \
+	    -font SidebarFont
+	ttk::style map SidebarButton \
+	    -foreground {
+		{disabled selected} systemWindowBackgroundColor3
+		{disabled !selected} systemDisabledControlTextColor
+		selected systemTextColor
+		active systemTextColor
+		pressed systemTextColor
+	    }
+
+	# For Entry, Combobox and Spinbox widgets the selected text background
+	# is the "Highlight color" selected in preferences when the widget
+	# has focus.  It is a gray color when the widget does not have focus or
+	# the window does not have focus. (The background state implies !focus
+	# so we only need to specify !focus.)
+
 	# Entry
 	ttk::style configure TEntry \
 	    -foreground systemTextColor \
@@ -43,13 +102,30 @@ namespace eval ttk::theme::aqua {
 	    -foreground {
 		disabled systemDisabledControlTextColor
 	    } \
-	    -selectforeground {
-		background systemTextColor
+	    -selectbackground {
+		!focus systemUnemphasizedSelectedTextBackgroundColor
+	    }
+
+	# Combobox:
+	ttk::style map TCombobox \
+	    -foreground {
+		disabled systemDisabledControlTextColor
 	    } \
 	    -selectbackground {
-		background systemTextBackgroundColor
+		!focus systemUnemphasizedSelectedTextBackgroundColor
+	    }
+
+	# Spinbox
+	ttk::style configure TSpinbox \
+	    -foreground systemTextColor \
+	    -background systemTextBackgroundColor
+	ttk::style map TSpinbox \
+	    -foreground {
+		disabled systemDisabledControlTextColor
+	    } \
+	    -selectbackground {
+		!focus systemUnemphasizedSelectedTextBackgroundColor
 	    }
-	    
 
 	# Workaround for #1100117:
 	# Actually, on Aqua we probably shouldn't stipple images in
@@ -63,52 +139,20 @@ namespace eval ttk::theme::aqua {
 	ttk::style configure TNotebook.Tab -foreground systemControlTextColor
 	ttk::style map TNotebook.Tab \
 	    -foreground {
-		background systemControlTextColor
-		disabled systemDisabledControlTextColor
-		selected systemSelectedTabTextColor}
+		{background !selected} systemControlTextColor
+		{background selected} black
+		{!background selected} systemSelectedTabTextColor
+		disabled systemDisabledControlTextColor}
 
-	# Combobox:
-	ttk::style configure TCombobox \
-	    -foreground systemTextColor \
-	    -background systemTransparent
-	ttk::style map TCombobox \
-	    -foreground {
-		disabled systemDisabledControlTextColor
-	    } \
-	    -selectforeground {
-		background systemTextColor
-	    } \
-	    -selectbackground {
-		background systemTransparent
-	    }
-
-	# Spinbox
-	ttk::style configure TSpinbox \
-	    -foreground systemTextColor \
-	    -background systemTextBackgroundColor \
-	    -selectforeground systemSelectedTextColor \
-	    -selectbackground systemSelectedTextBackgroundColor
-	ttk::style map TSpinbox \
-	    -foreground {
-		disabled systemDisabledControlTextColor
-	    } \
-	    -selectforeground {
-		!active systemTextColor
-	    } \
-	    -selectbackground {
-		!active systemTextBackgroundColor
-		!focus systemTextBackgroundColor
-		focus systemSelectedTextBackgroundColor
-	    }
-	
 	# Treeview:
 	ttk::style configure Heading \
 	    -font TkHeadingFont \
 	    -foreground systemTextColor \
 	    -background systemWindowBackgroundColor
 	ttk::style configure Treeview -rowheight 18 \
-	    -background systemTextBackgroundColor \
-	    -foreground systemTextColor \
+	    -background systemControlBackgroundColor \
+	    -stripedbackground systemControlAlternatingRowColor \
+            -foreground systemTextColor \
 	    -fieldbackground systemTextBackgroundColor
 	ttk::style map Treeview \
 	    -background {
@@ -116,14 +160,18 @@ namespace eval ttk::theme::aqua {
 	    }
 
 	# Enable animation for ttk::progressbar widget:
-	ttk::style configure TProgressbar -period 100 -maxphase 255
+	ttk::style configure TProgressbar -period 100 -maxphase 120
 
 	# For Aqua, labelframe labels should appear outside the border,
 	# with a 14 pixel inset and 4 pixels spacing between border and label
 	# (ref: Apple Human Interface Guidelines / Controls / Grouping Controls)
 	#
 	ttk::style configure TLabelframe \
-		-labeloutside true -labelmargins {14 0 14 4}
+	    -labeloutside true \
+	    -labelmargins {14 0 14 2}
+
+	ttk::style configure TLabelframe.Label \
+	    -font TkSmallCaptionFont
 
 	# TODO: panedwindow sashes should be 9 pixels (HIG:Controls:Split Views)
     }
