@@ -14,34 +14,8 @@
  */
 
 #include "tkMacOSXPrivate.h"
-#include "tkMacOSXEvent.h"
+#include "tkMacOSXInt.h"
 #include "tkMacOSXConstants.h"
-
-#ifdef USE_TCL_STUBS
-#ifdef __cplusplus
-extern "C" {
-#endif
-/*  Little hack to eliminate the need for "tclInt.h" here:
-    Just copy a small portion of TclIntPlatStubs, just
-    enough to make it work. See [600b72bfbc] */
-typedef struct {
-    int magic;
-    void *hooks;
-    void (*dummy[19]) (void); /* dummy entries 0-18, not used */
-    void (*tclMacOSXNotifierAddRunLoopMode) (const void *runLoopMode); /* 19 */
-} TclIntPlatStubs;
-extern const TclIntPlatStubs *tclIntPlatStubsPtr;
-#ifdef __cplusplus
-}
-#endif
-#define TclMacOSXNotifierAddRunLoopMode \
-	(tclIntPlatStubsPtr->tclMacOSXNotifierAddRunLoopMode) /* 19 */
-#elif TCL_MINOR_VERSION < 7
-    extern void TclMacOSXNotifierAddRunLoopMode(const void *runLoopMode);
-#else
-    extern void Tcl_MacOSXNotifierAddRunLoopMode(const void *runLoopMode);
-#   define TclMacOSXNotifierAddRunLoopMode Tcl_MacOSXNotifierAddRunLoopMode
-#endif
 #import <objc/objc-auto.h>
 
 /* This is not used for anything at the moment. */
@@ -53,9 +27,9 @@ static Tcl_ThreadDataKey dataKey;
 #define TSD_INIT() ThreadSpecificData *tsdPtr = (ThreadSpecificData *) \
 	Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData))
 
-static void TkMacOSXNotifyExitHandler(ClientData clientData);
-static void TkMacOSXEventsSetupProc(ClientData clientData, int flags);
-static void TkMacOSXEventsCheckProc(ClientData clientData, int flags);
+static void TkMacOSXNotifyExitHandler(void *clientData);
+static void TkMacOSXEventsSetupProc(void *clientData, int flags);
+static void TkMacOSXEventsCheckProc(void *clientData, int flags);
 
 #ifdef TK_MAC_DEBUG_EVENTS
 static const char *Tk_EventName[39] = {
@@ -102,7 +76,7 @@ static const char *Tk_EventName[39] = {
 
 static Tk_RestrictAction
 InspectQueueRestrictProc(
-     ClientData arg,
+     void *arg,
      XEvent *eventPtr)
 {
     XVirtualEvent* ve = (XVirtualEvent*) eventPtr;
@@ -126,7 +100,7 @@ InspectQueueRestrictProc(
 
 void DebugPrintQueue(void)
 {
-    ClientData oldArg;
+    void *oldArg;
     Tk_RestrictProc *oldProc;
 
     oldProc = Tk_RestrictEvents(InspectQueueRestrictProc, NULL, &oldArg);
@@ -181,7 +155,7 @@ void DebugPrintQueue(void)
      * this block should be removed.
      */
 
-# if MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101500
     if ([theEvent type] == NSAppKitDefined) {
 	static Bool aWindowIsMoving = NO;
 	switch([theEvent subtype]) {
@@ -294,8 +268,8 @@ Tk_MacOSXSetupTkNotifier(void)
 	    Tcl_CreateEventSource(TkMacOSXEventsSetupProc,
 		    TkMacOSXEventsCheckProc, NULL);
 	    TkCreateExitHandler(TkMacOSXNotifyExitHandler, NULL);
-	    TclMacOSXNotifierAddRunLoopMode(NSEventTrackingRunLoopMode);
-	    TclMacOSXNotifierAddRunLoopMode(NSModalPanelRunLoopMode);
+	    Tcl_MacOSXNotifierAddRunLoopMode(NSEventTrackingRunLoopMode);
+	    Tcl_MacOSXNotifierAddRunLoopMode(NSModalPanelRunLoopMode);
 	}
     }
 }
@@ -319,7 +293,7 @@ Tk_MacOSXSetupTkNotifier(void)
 
 static void
 TkMacOSXNotifyExitHandler(
-    ClientData clientData)	/* Not used. */
+    TCL_UNUSED(void *))	/* Not used. */
 {
     TSD_INIT();
 
@@ -336,12 +310,12 @@ TkMacOSXNotifyExitHandler(
  *       This static function is meant to be run as an idle task.  It attempts
  *       to redraw all views which have the tkNeedsDisplay property set to YES.
  *       This relies on a feature of [NSApp nextEventMatchingMask: ...] which
- *       is undocumented, namely that it sometimes blocks and calls drawRect
+ *       is undocumented, namely that it sometimes blocks and calls updateLayer
  *       for all views that need display before it returns.  We call it with
  *       deQueue=NO so that it will not change anything on the AppKit event
  *       queue, because we only want the side effect that it runs drawRect. The
  *       only times when any NSViews have the needsDisplay property set to YES
- *       are during execution of this function or in the addTkDirtyRect method
+ *       are during execution of this function or in the setFrameSize method
  *       of TKContentView.
  *
  *       The reason for running this function as an idle task is to try to
@@ -366,9 +340,9 @@ TkMacOSXNotifyExitHandler(
 
 void
 TkMacOSXDrawAllViews(
-    ClientData clientData)
+    void *clientData)
 {
-       int count = 0, *dirtyCount = (int *)clientData;
+    int count = 0, *dirtyCount = (int *)clientData;
 
     for (NSWindow *window in [NSApp windows]) {
 	if ([[window contentView] isMemberOfClass:[TKContentView class]]) {
@@ -378,7 +352,6 @@ TkMacOSXDrawAllViews(
 		if (dirtyCount) {
 		   continue;
 		}
-		[[view layer] setNeedsDisplayInRect:NSRectToCGRect([view tkDirtyRect])];
 		[view setNeedsDisplay:YES];
 	    }
 	} else {
@@ -388,26 +361,15 @@ TkMacOSXDrawAllViews(
     if (dirtyCount) {
     	*dirtyCount = count;
     }
+
+    /*
+     * Trigger calls to updateLayer methods for the views flagged above.
+     */
+
     [NSApp nextEventMatchingMask:NSAnyEventMask
 		       untilDate:[NSDate distantPast]
 			  inMode:GetRunLoopMode(TkMacOSXGetModalSession())
 			 dequeue:NO];
-    for (NSWindow *window in [NSApp windows]) {
-	if ([[window contentView] isMemberOfClass:[TKContentView class]]) {
-	    TKContentView *view = [window contentView];
-
-	    /*
-	     * If we did not run drawRect, we set needsDisplay back to NO.
-	     * Note that if drawRect did run it may have added to Tk's dirty
-	     * rect, due to attempts to draw outside of drawRect's dirty rect.
-	     */
-
-	    if ([view needsDisplay]) {
-		[view setNeedsDisplay: NO];
-	    }
-	}
-    }
-    [NSApp setNeedsToDraw:NO];
 }
 
 /*
@@ -438,7 +400,7 @@ static Tcl_TimerToken ticker = NULL;
 
 static void
 Heartbeat(
-    ClientData clientData)
+    TCL_UNUSED(void *))
 {
 
     if (ticker) {
@@ -450,7 +412,7 @@ static const Tcl_Time zeroBlockTime = { 0, 0 };
 
 static void
 TkMacOSXEventsSetupProc(
-    ClientData clientData,
+    TCL_UNUSED(void *),
     int flags)
 {
     NSString *runloopMode = [[NSRunLoop currentRunLoop] currentMode];
@@ -474,11 +436,11 @@ TkMacOSXEventsSetupProc(
   	 */
 
 	NSEvent *currentEvent =
-	        [NSApp nextEventMatchingMask:NSAnyEventMask
+		[NSApp nextEventMatchingMask:NSAnyEventMask
 			untilDate:[NSDate distantPast]
 			inMode:GetRunLoopMode(TkMacOSXGetModalSession())
 			dequeue:NO];
-	if ((currentEvent) || [NSApp needsToDraw] ) {
+	if ((currentEvent)) {
 	    Tcl_SetMaxBlockTime(&zeroBlockTime);
 	    Tcl_DeleteTimerHandler(ticker);
 	    ticker = NULL;
@@ -516,7 +478,7 @@ TkMacOSXEventsSetupProc(
  */
 static void
 TkMacOSXEventsCheckProc(
-    ClientData clientData,
+    TCL_UNUSED(void *),
     int flags)
 {
     NSString *runloopMode = [[NSRunLoop currentRunLoop] currentMode];
