@@ -95,7 +95,7 @@ static ScrollbarMetrics metrics = {
  * Declarations of static functions defined later in this file:
  */
 
-static void		ScrollbarEventProc(ClientData clientData,
+static void		ScrollbarEventProc(void *clientData,
 			    XEvent *eventPtr);
 static int		ScrollbarEvent(TkScrollbar *scrollPtr,
 			    XEvent *eventPtr);
@@ -185,7 +185,7 @@ static void drawMacScrollbar(
     if (scrollPtr->vertical) {
 	thumbOrigin.x = troughBounds.origin.x + MIN_GAP;
 	thumbOrigin.y = troughBounds.origin.y + scrollPtr->sliderFirst;
-	thumbSize.width = troughBounds.size.width - 2*MIN_GAP + 1;
+	thumbSize.width = troughBounds.size.width - 2 * MIN_GAP + 1;
 	thumbSize.height = scrollPtr->sliderLast - scrollPtr->sliderFirst;
 	inner[0] = troughBounds.origin;
 	inner[1] = CGPointMake(inner[0].x,
@@ -197,7 +197,7 @@ static void drawMacScrollbar(
 	thumbOrigin.x = troughBounds.origin.x + scrollPtr->sliderFirst;
 	thumbOrigin.y = troughBounds.origin.y + MIN_GAP;
 	thumbSize.width = scrollPtr->sliderLast - scrollPtr->sliderFirst;
-	thumbSize.height = troughBounds.size.height - 2*MIN_GAP + 1;
+	thumbSize.height = troughBounds.size.height - 2 * MIN_GAP + 1;
 	inner[0] = troughBounds.origin;
 	inner[1] = CGPointMake(inner[0].x + troughBounds.size.width,
 			       inner[0].y + 1);
@@ -244,13 +244,14 @@ static void drawMacScrollbar(
 
 void
 TkpDisplayScrollbar(
-    ClientData clientData)	/* Information about window. */
+    void *clientData)	/* Information about window. */
 {
     TkScrollbar *scrollPtr = (TkScrollbar *)clientData;
     MacScrollbar *msPtr = (MacScrollbar *) scrollPtr;
     Tk_Window tkwin = scrollPtr->tkwin;
     TkWindow *winPtr = (TkWindow *) tkwin;
     TkMacOSXDrawingContext dc;
+    int borderWidth, highlightWidth;
 
     scrollPtr->flags &= ~REDRAW_PENDING;
 
@@ -262,7 +263,6 @@ TkpDisplayScrollbar(
     NSView *view = TkMacOSXGetNSViewForDrawable(macWin);
 
     if ((view == NULL)
-	    || (macWin->flags & TK_DO_NOT_DRAW)
 	    || !TkMacOSXSetupDrawingContext((Drawable)macWin, NULL, &dc)) {
 	return;
     }
@@ -284,7 +284,8 @@ TkpDisplayScrollbar(
      * Draw a 3D rectangle to provide a base for the native scrollbar.
      */
 
-    if (scrollPtr->highlightWidth != 0) {
+    Tk_GetPixelsFromObj(NULL, scrollPtr->tkwin, scrollPtr->highlightWidthObj, &highlightWidth);
+    if (highlightWidth > 0) {
     	GC fgGC, bgGC;
 
     	bgGC = Tk_GCForColor(scrollPtr->highlightBgColorPtr, (Pixmap) macWin);
@@ -293,19 +294,20 @@ TkpDisplayScrollbar(
     	} else {
     	    fgGC = bgGC;
     	}
-    	TkpDrawHighlightBorder(tkwin, fgGC, bgGC, scrollPtr->highlightWidth,
+    	Tk_DrawHighlightBorder(tkwin, fgGC, bgGC, highlightWidth,
     		(Pixmap) macWin);
     }
 
+    Tk_GetPixelsFromObj(NULL, scrollPtr->tkwin, scrollPtr->borderWidthObj, &borderWidth);
     Tk_Draw3DRectangle(tkwin, (Pixmap) macWin, scrollPtr->bgBorder,
-	    scrollPtr->highlightWidth, scrollPtr->highlightWidth,
-	    Tk_Width(tkwin) - 2*scrollPtr->highlightWidth,
-	    Tk_Height(tkwin) - 2*scrollPtr->highlightWidth,
-	    scrollPtr->borderWidth, scrollPtr->relief);
+	    highlightWidth, highlightWidth,
+	    Tk_Width(tkwin) - 2 * highlightWidth,
+	    Tk_Height(tkwin) - 2 * highlightWidth,
+	    borderWidth, scrollPtr->relief);
     Tk_Fill3DRectangle(tkwin, (Pixmap) macWin, scrollPtr->bgBorder,
 	    scrollPtr->inset, scrollPtr->inset,
-	    Tk_Width(tkwin) - 2*scrollPtr->inset,
-	    Tk_Height(tkwin) - 2*scrollPtr->inset, 0, TK_RELIEF_FLAT);
+	    Tk_Width(tkwin) - 2 * scrollPtr->inset,
+	    Tk_Height(tkwin) - 2 * scrollPtr->inset, 0, TK_RELIEF_FLAT);
 
     /*
      * Update values and then draw the native scrollbar over the rectangle.
@@ -372,19 +374,20 @@ TkpComputeScrollbarGeometry(
      */
 
     int fieldLength;
+    int width, borderWidth, highlightWidth;
 
-    if (scrollPtr->highlightWidth < 0) {
-	scrollPtr->highlightWidth = 0;
-    }
-    scrollPtr->inset = scrollPtr->highlightWidth + scrollPtr->borderWidth;
+    Tk_GetPixelsFromObj(NULL, scrollPtr->tkwin, scrollPtr->borderWidthObj, &borderWidth);
+    Tk_GetPixelsFromObj(NULL, scrollPtr->tkwin, scrollPtr->widthObj, &width);
+    Tk_GetPixelsFromObj(NULL, scrollPtr->tkwin, scrollPtr->highlightWidthObj, &highlightWidth);
+    scrollPtr->inset = highlightWidth + borderWidth;
     if ([NSApp macOSVersion] == 100600) {
-	scrollPtr->arrowLength = scrollPtr->width;
+	scrollPtr->arrowLength = width;
     } else {
 	scrollPtr->arrowLength = 0;
     }
     fieldLength = (scrollPtr->vertical ? Tk_Height(scrollPtr->tkwin)
 	    : Tk_Width(scrollPtr->tkwin))
-	    - 2*(scrollPtr->arrowLength + scrollPtr->inset);
+	    - 2 * (scrollPtr->arrowLength + scrollPtr->inset);
     if (fieldLength < 0) {
 	fieldLength = 0;
     }
@@ -422,14 +425,14 @@ TkpComputeScrollbarGeometry(
 
     if (scrollPtr->vertical) {
 	Tk_GeometryRequest(scrollPtr->tkwin,
-		scrollPtr->width + 2*scrollPtr->inset,
-		2*(scrollPtr->arrowLength + scrollPtr->borderWidth
+		width + 2 * scrollPtr->inset,
+		2 * (scrollPtr->arrowLength + borderWidth
 		+ scrollPtr->inset) + metrics.minThumbHeight);
     } else {
 	Tk_GeometryRequest(scrollPtr->tkwin,
-		2*(scrollPtr->arrowLength + scrollPtr->borderWidth
+		2 * (scrollPtr->arrowLength + borderWidth
 		+ scrollPtr->inset) + metrics.minThumbHeight,
-		scrollPtr->width + 2*scrollPtr->inset);
+		width + 2 * scrollPtr->inset);
     }
     Tk_SetInternalBorder(scrollPtr->tkwin, scrollPtr->inset);
 }
@@ -484,9 +487,8 @@ TkpDestroyScrollbar(
 
 void
 TkpConfigureScrollbar(
-    TkScrollbar *scrollPtr)
+    TCL_UNUSED(TkScrollbar *))
 {
-    (void)scrollPtr;
     /* empty */
 }
 
@@ -551,7 +553,7 @@ TkpScrollbarPosition(
     if (y < scrollPtr->sliderLast) {
 	return SLIDER;
     }
-    if (y < length - (2*scrollPtr->arrowLength + inset)) {
+    if (y < length - (2 * scrollPtr->arrowLength + inset)) {
 	return BOTTOM_GAP;
     }
 
@@ -650,8 +652,8 @@ UpdateControlValues(
 	    || height <= metrics.minHeight) {
     	msPtr->info.enableState = kThemeTrackHideTrack;
     } else {
-        msPtr->info.enableState = kThemeTrackActive;
-    	msPtr->info.attributes =
+	msPtr->info.enableState = kThemeTrackActive;
+	msPtr->info.attributes =
 		kThemeTrackShowThumb | kThemeTrackThumbRgnIsNotGhost;
     }
 }
@@ -766,7 +768,7 @@ ScrollbarEvent(
 
 static void
 ScrollbarEventProc(
-    ClientData clientData,	/* Information about window. */
+    void *clientData,	/* Information about window. */
     XEvent *eventPtr)		/* Information about event. */
 {
     TkScrollbar *scrollPtr = (TkScrollbar *)clientData;
