@@ -47,7 +47,6 @@ typedef struct Transient {
 } Transient;
 
 #define WITHDRAWN_BY_CONTAINER 0x1
-#define WITHDRAWN_BY_MASTER 0x1
 
 /*
  * A data structure of the following type holds window-manager-related
@@ -172,7 +171,7 @@ typedef struct TkWmInfo {
      */
 
     TkWindow **cmapList;	/* Array of window with private colormaps. */
-    int cmapCount;		/* Number of windows in array. */
+    Tcl_Size cmapCount;		/* Number of windows in array. */
 
     /*
      * Miscellaneous information.
