@@ -5,7 +5,7 @@ if {[namespace exists tk::test]} {
     return
 }
 
-package require Tk
+package require tk
 tk appname tktest
 wm title . tktest
 # If the main window isn't already mapped (e.g. because the tests are
@@ -148,7 +148,7 @@ namespace eval tk {
         proc imageInit {} {
             variable ImageNames
             if {![info exists ImageNames]} {
-                set ImageNames [lsort [image names]]
+                set ImageNames [lsearch -all -inline -glob -not [lsort [image names]] ::tk::icons::indicator*]
             }
             imageCleanup
             if {[lsort [image names]] ne $ImageNames} {
@@ -157,7 +157,8 @@ namespace eval tk {
         }
         proc imageFinish {} {
             variable ImageNames
-            if {[lsort [image names]] ne $ImageNames} {
+	    set imgs [lsearch -all -inline -glob -not [lsort [image names]] ::tk::icons::indicator*]
+            if {$imgs ne $ImageNames} {
                 return -code error "images remaining: [image names] != $ImageNames"
             }
             imageCleanup
@@ -188,13 +189,8 @@ namespace eval tk {
 	# It takes care of the following timing details of pointer warping:
 	#
 	# a. Allow pointer warping to happen if it was scheduled for execution at
-	#    idle time.
-	#    - In Tk releases 8.6 and older, pointer warping is scheduled for
-	#      execution at idle time
-	#    - In release 8.7 and newer this happens synchronously if $w refers to the
-	#      whole screen or if the -when option to [event generate] is "now".
-	#    The namespace variable idle_pointer_warping records which of these is
-	#    the case.
+	#    idle time. This happens synchronously if $w refers to the
+	#    whole screen or if the -when option to [event generate] is "now".
 	#
 	# b. Work around a race condition associated with OS notification of
 	#    mouse motion on Windows.
@@ -237,12 +233,8 @@ namespace eval tk {
 	# to [event generate $w] is not "now", and $w refers to a Tk window, i.e. not
 	# the whole screen.
 	#
-	variable idle_pointer_warping [expr {![package vsatisfies [package provide Tk] 8.7-]}]
 	proc controlPointerWarpTiming {{duration 50}} {
-		variable idle_pointer_warping
-		if {$idle_pointer_warping} {
-			update idletasks ;# see a. above
-		}
+		update idletasks ;# see a. above
 		if {[tk windowingsystem] eq "win32"} {
 			after $duration ;# see b. above
 		}
@@ -272,6 +264,8 @@ testConstraint altDisplay  [info exists env(TK_ALT_DISPLAY)]
 testConstraint noExceed [expr {
     ![testConstraint unix] || [catch {font actual "\{xyz"}]
 }]
+testConstraint deprecated [expr {![::tk::build-info no-deprecate]}]
+
 # constraint for running a test on all windowing system except aqua
 # where the test fails due to a known bug
 testConstraint aquaKnownBug [expr {[testConstraint notAqua] || [testConstraint knownBug]}]
@@ -394,4 +388,3 @@ namespace import -force tcltest::cleanupTests
 deleteWindows
 wm geometry . {}
 raise .
-
